@@ -117,6 +117,14 @@ int fossil_isalnum(char c){
   return (c>='a' && c<='z') || (c>='A' && c<='Z') || (c>='0' && c<='9');
 }
 
+/* Return true if and only if the entire string consists of only
+** alphanumeric characters.
+*/
+int fossil_no_strange_characters(const char *z){
+  while( z && (fossil_isalnum(z[0]) || z[0]=='_' || z[0]=='-') ) z++;
+  return z[0]==0;
+}
+
 
 /*
 ** COMMAND: test-isspace
@@ -271,14 +279,29 @@ void blob_zero(Blob *pBlob){
 
 /*
 ** Append text or data to the end of a blob.
+**
+** The blob_append_full() routine is a complete implementation.
+** The blob_append() routine only works for cases where nData>0 and
+** no resizing is required, and falls back to blob_append_full() if
+** either condition is not met, but runs faster in the common case
+** where all conditions are met.  The use of blob_append() is
+** recommended, unless it is known in advance that nData<0.
 */
-void blob_append(Blob *pBlob, const char *aData, int nData){
+void blob_append_full(Blob *pBlob, const char *aData, int nData){
+  sqlite3_int64 nNew;
   assert( aData!=0 || nData==0 );
   blob_is_init(pBlob);
   if( nData<0 ) nData = strlen(aData);
   if( nData==0 ) return;
-  if( pBlob->nUsed + nData >= pBlob->nAlloc ){
-    pBlob->xRealloc(pBlob, pBlob->nUsed + nData + pBlob->nAlloc + 100);
+  nNew = pBlob->nUsed;
+  nNew += nData;
+  if( nNew >= pBlob->nAlloc ){
+    nNew += pBlob->nAlloc;
+    nNew += 100;
+    if( nNew>=0x7fff0000 ){
+      blob_panic();
+    }
+    pBlob->xRealloc(pBlob, (int)nNew);
     if( pBlob->nUsed + nData >= pBlob->nAlloc ){
       blob_panic();
     }
@@ -287,6 +310,37 @@ void blob_append(Blob *pBlob, const char *aData, int nData){
   pBlob->nUsed += nData;
   pBlob->aData[pBlob->nUsed] = 0;   /* Blobs are always nul-terminated */
 }
+void blob_append(Blob *pBlob, const char *aData, int nData){
+  sqlite3_int64 nUsed;
+  assert( aData!=0 || nData==0 );
+  /* blob_is_init(pBlob); // omitted for speed */
+  if( nData<=0 || pBlob->nUsed + nData >= pBlob->nAlloc ){
+    blob_append_full(pBlob, aData, nData);
+    return;
+  }
+  nUsed = pBlob->nUsed;
+  pBlob->nUsed += nData;
+  pBlob->aData[pBlob->nUsed] = 0;
+  memcpy(&pBlob->aData[nUsed], aData, nData);
+}
+
+/*
+** Append a string literal to a blob.
+*/
+#if INTERFACE
+#define blob_append_string(BLOB,STR) blob_append(BLOB,STR,sizeof(STR)-1)
+#endif
+
+/*
+** Append a single character to the blob
+*/
+void blob_append_char(Blob *pBlob, char c){
+  if( pBlob->nUsed+1 >= pBlob->nAlloc ){
+    blob_append_full(pBlob, &c, 1);
+  }else{
+    pBlob->aData[pBlob->nUsed++] = c;
+  }
+}
 
 /*
 ** Copy a blob
@@ -303,10 +357,12 @@ void blob_copy(Blob *pTo, Blob *pFrom){
 char *blob_str(Blob *p){
   blob_is_init(p);
   if( p->nUsed==0 ){
-    blob_append(p, "", 1); /* NOTE: Changes nUsed. */
+    blob_append_char(p, 0); /* NOTE: Changes nUsed. */
     p->nUsed = 0;
   }
-  if( p->aData[p->nUsed]!=0 ){
+  if( p->nUsed<p->nAlloc ){
+    p->aData[p->nUsed] = 0;
+  }else{
     blob_materialize(p);
   }
   return p->aData;
@@ -320,7 +376,7 @@ char *blob_str(Blob *p){
 char *blob_sql_text(Blob *p){
   blob_is_init(p);
   if( (p->blobFlags & BLOBFLAG_NotSQL) ){
-    fossil_fatal("Internal error: Use of blob_appendf() to construct SQL text");
+    fossil_panic("use of blob_appendf() to construct SQL text");
   }
   return blob_str(p);
 }
@@ -424,6 +480,30 @@ void blob_resize(Blob *pBlob, unsigned int newSize){
   pBlob->aData[newSize] = 0;
 }
 
+/*
+** Ensures that the given blob has at least the given amount of memory
+** allocated to it. Does not modify pBlob->nUsed nor will it reduce
+** the currently-allocated amount of memory.
+**
+** For semantic compatibility with blob_append_full(), if newSize is
+** >=0x7fff000 (~2GB) then this function will trigger blob_panic(). If
+** it didn't, it would be possible to bypass that hard-coded limit via
+** this function.
+**
+** We've had at least one report:
+**   https://fossil-scm.org/forum/forumpost/b7bbd28db4
+** which implies that this is unconditionally failing on mingw 32-bit
+** builds.
+*/
+void blob_reserve(Blob *pBlob, unsigned int newSize){
+  if(newSize>=0x7fff0000 ){
+    blob_panic();
+  }else if(newSize>pBlob->nUsed){
+    pBlob->xRealloc(pBlob, newSize);
+    pBlob->aData[newSize] = 0;
+  }
+}
+
 /*
 ** Make sure a blob is nul-terminated and is not a pointer to unmanaged
 ** space.  Return a pointer to the data.
@@ -475,6 +555,13 @@ void blob_rewind(Blob *p){
   p->iCursor = 0;
 }
 
+/*
+** Truncate a blob back to zero length
+*/
+void blob_truncate(Blob *p, int sz){
+  if( sz>=0 && sz<p->nUsed ) p->nUsed = sz;
+}
+
 /*
 ** Seek the cursor in a blob to the indicated offset.
 */
@@ -648,6 +735,16 @@ void blob_copy_lines(Blob *pTo, Blob *pFrom, int N){
   pFrom->iCursor = i;
 }
 
+/*
+** Ensure that the text in pBlob ends with '\n'
+*/
+void blob_add_final_newline(Blob *pBlob){
+  if( pBlob->nUsed<=0 ) return;
+  if( pBlob->aData[pBlob->nUsed-1]!='\n' ){
+    blob_append_char(pBlob, '\n');
+  }
+}
+
 /*
 ** Return true if the blob contains a valid base16 identifier artifact hash.
 **
@@ -786,19 +883,37 @@ int blob_read_from_channel(Blob *pBlob, FILE *in, int nToRead){
 ** Initialize a blob to be the content of a file.  If the filename
 ** is blank or "-" then read from standard input.
 **
+** If zFilename is a symbolic link, behavior depends on the eFType
+** parameter:
+**
+**    *  If eFType is ExtFILE or allow-symlinks is OFF, then the
+**       pBlob is initialized to the *content* of the object to which
+**       the zFilename symlink points.
+**
+**    *  If eFType is RepoFILE and allow-symlinks is ON, then the
+**       pBlob is initialized to the *name* of the object to which
+**       the zFilename symlink points.
+**
 ** Any prior content of the blob is discarded, not freed.
 **
 ** Return the number of bytes read. Calls fossil_fatal() on error (i.e.
 ** it exit()s and does not return).
 */
-sqlite3_int64 blob_read_from_file(Blob *pBlob, const char *zFilename){
+sqlite3_int64 blob_read_from_file(
+  Blob *pBlob,               /* The blob to be initialized */
+  const char *zFilename,     /* Extract content from this file */
+  int eFType                 /* ExtFILE or RepoFILE - see above */
+){
   sqlite3_int64 size, got;
   FILE *in;
   if( zFilename==0 || zFilename[0]==0
         || (zFilename[0]=='-' && zFilename[1]==0) ){
     return blob_read_from_channel(pBlob, stdin, -1);
   }
-  size = file_wd_size(zFilename);
+  if( file_islink(zFilename) ){
+    return blob_read_link(pBlob, zFilename);
+  }
+  size = file_size(zFilename, eFType);
   blob_zero(pBlob);
   if( size<0 ){
     fossil_fatal("no such file: %s", zFilename);
@@ -844,12 +959,16 @@ int blob_read_link(Blob *pBlob, const char *zFilename){
 #endif
 }
 
-
 /*
 ** Write the content of a blob into a file.
 **
 ** If the filename is blank or "-" then write to standard output.
 **
+** This routine always assumes ExtFILE.  If zFilename is a symbolic link
+** then the content is written into the object that symbolic link points
+** to, not into the symbolic link itself.  This is true regardless of
+** the allow-symlinks setting.
+**
 ** Return the number of bytes written.
 */
 int blob_write_to_file(Blob *pBlob, const char *zFilename){
@@ -870,7 +989,7 @@ int blob_write_to_file(Blob *pBlob, const char *zFilename){
     _setmode(_fileno(stdout), _O_TEXT);
 #endif
   }else{
-    file_mkfolder(zFilename, 1, 0);
+    file_mkfolder(zFilename, ExtFILE, 1, 0);
     out = fossil_fopen(zFilename, "wb");
     if( out==0 ){
 #if _WIN32
@@ -935,7 +1054,7 @@ void blob_compress(Blob *pIn, Blob *pOut){
 void compress_cmd(void){
   Blob f;
   if( g.argc!=4 ) usage("INPUTFILE OUTPUTFILE");
-  blob_read_from_file(&f, g.argv[2]);
+  blob_read_from_file(&f, g.argv[2], ExtFILE);
   blob_compress(&f, &f);
   blob_write_to_file(&f, g.argv[3]);
 }
@@ -994,8 +1113,8 @@ void blob_compress2(Blob *pIn1, Blob *pIn2, Blob *pOut){
 void compress2_cmd(void){
   Blob f1, f2;
   if( g.argc!=5 ) usage("INPUTFILE1 INPUTFILE2 OUTPUTFILE");
-  blob_read_from_file(&f1, g.argv[2]);
-  blob_read_from_file(&f2, g.argv[3]);
+  blob_read_from_file(&f1, g.argv[2], ExtFILE);
+  blob_read_from_file(&f2, g.argv[3], ExtFILE);
   blob_compress2(&f1, &f2, &f1);
   blob_write_to_file(&f1, g.argv[4]);
 }
@@ -1046,7 +1165,7 @@ int blob_uncompress(Blob *pIn, Blob *pOut){
 void uncompress_cmd(void){
   Blob f;
   if( g.argc!=4 ) usage("INPUTFILE OUTPUTFILE");
-  blob_read_from_file(&f, g.argv[2]);
+  blob_read_from_file(&f, g.argv[2], ExtFILE);
   blob_uncompress(&f, &f);
   blob_write_to_file(&f, g.argv[3]);
 }
@@ -1061,7 +1180,7 @@ void test_cycle_compress(void){
   int i;
   Blob b1, b2, b3;
   for(i=2; i<g.argc; i++){
-    blob_read_from_file(&b1, g.argv[i]);
+    blob_read_from_file(&b1, g.argv[i], ExtFILE);
     blob_compress(&b1, &b2);
     blob_uncompress(&b2, &b3);
     if( blob_compare(&b1, &b3) ){
@@ -1074,7 +1193,6 @@ void test_cycle_compress(void){
   fossil_print("ok\n");
 }
 
-#if defined(_WIN32) || defined(__CYGWIN__)
 /*
 ** Convert every \n character in the given blob into \r\n.
 */
@@ -1098,7 +1216,6 @@ void blob_add_cr(Blob *p){
     }
   }
 }
-#endif
 
 /*
 ** Remove every \r character from the given blob, replacing each one with
@@ -1191,30 +1308,84 @@ void blob_append_escaped_arg(Blob *pBlob, const char *zIn){
   int n = blob_size(pBlob);
   char *z = blob_buffer(pBlob);
 #if defined(_WIN32)
+  const char cDirSep = '\\';  /* Use \ as directory separator */
   const char cQuote = '"';    /* Use "..." quoting on windows */
+  const char cEscape = '^';   /* Use ^X escaping on windows */
 #else
+  const char cDirSep = '/';   /* Use / as directory separator */
   const char cQuote = '\'';   /* Use '...' quoting on unix */
+  const char cEscape = '\\';  /* Use \X escaping on unix */
 #endif
 
   for(i=0; (c = zIn[i])!=0; i++){
-    if( c==cQuote || c=='\\' || c<' ' || c==';' || c=='*' || c=='?' || c=='[') {
+    if( c==cQuote || (unsigned char)c<' ' ||
+        c==cEscape || c==';' || c=='*' || c=='?' || c=='[' ){
       Blob bad;
       blob_token(pBlob, &bad);
       fossil_fatal("the [%s] argument to the \"%s\" command contains "
                    "a character (ascii 0x%02x) that is a security risk",
                    zIn, blob_str(&bad), c);
     }
-    if( !needEscape && !fossil_isalnum(c) && c!='/' && c!='.' && c!='_' ){
+    if( !needEscape && !fossil_isalnum(c) && c!=cDirSep && c!='.' && c!='_' ){
       needEscape = 1;
     }
   }
   if( n>0 && !fossil_isspace(z[n-1]) ){
-    blob_append(pBlob, " ", 1);
+    blob_append_char(pBlob, ' ');
   }
-  if( needEscape ) blob_append(pBlob, &cQuote, 1);
-  if( zIn[0]=='-' ) blob_append(pBlob, "./", 2);
+  if( needEscape ) blob_append_char(pBlob, cQuote);
+  if( zIn[0]=='-' ){
+    blob_append_char(pBlob, '.');
+    blob_append_char(pBlob, cDirSep);
+#if defined(_WIN32)
+  }else if( zIn[0]=='/' ){
+    blob_append_char(pBlob, '.');
+#endif
+  }
+#if defined(_WIN32)
+  if( needEscape ){
+    for(i=0; (c = zIn[i])!=0; i++){
+      if( c==cQuote ) blob_append_char(pBlob, cDirSep);
+      blob_append_char(pBlob, c);
+    }
+  }else{
+    blob_append(pBlob, zIn, -1);
+  }
+#else
   blob_append(pBlob, zIn, -1);
-  if( needEscape ) blob_append(pBlob, &cQuote, 1);
+#endif
+  if( needEscape ){
+#if defined(_WIN32)
+    /* NOTE: Trailing backslash must be doubled before final double quote. */
+    if( pBlob->aData[pBlob->nUsed-1]==cDirSep ){
+      blob_append_char(pBlob, cDirSep);
+    }
+#endif
+    blob_append_char(pBlob, cQuote);
+  }
+}
+
+/*
+** COMMAND: test-escaped-arg
+**
+** Usage %fossil ARG ...
+**
+** Run each argment through blob_append_escaped_arg() and show the
+** result.  Append each argument to "fossil test-echo" and run that
+** using fossil_system() to verify that it really does get escaped
+** correctly.
+*/
+void test_escaped_arg__cmd(void){
+  int i;
+  Blob x;
+  blob_init(&x, 0, 0);
+  for(i=2; i<g.argc; i++){
+    fossil_print("%3d [%s]: ", i, g.argv[i]);
+    blob_appendf(&x, "fossil test-echo %$", g.argv[i]);
+    fossil_print("%s\n", blob_str(&x));
+    fossil_system(blob_str(&x));
+    blob_reset(&x);
+  }
 }
 
 /*
@@ -1277,7 +1448,7 @@ void blob_to_utf8_no_bom(Blob *pBlob, int useMbcs){
     if( bomReverse ){
       /* Found BOM, but with reversed bytes */
       unsigned int i = blob_size(pBlob);
-      while( i>0 ){
+      while( i>1 ){
         /* swap bytes of unicode representation */
         char zTemp = zUtf8[--i];
         zUtf8[i] = zUtf8[i-1];
@@ -1285,9 +1456,10 @@ void blob_to_utf8_no_bom(Blob *pBlob, int useMbcs){
       }
     }
     /* Make sure the blob contains two terminating 0-bytes */
-    blob_append(pBlob, "", 1);
+    blob_append(pBlob, "\000\000", 3);
     zUtf8 = blob_str(pBlob) + bomSize;
     zUtf8 = fossil_unicode_to_utf8(zUtf8);
+    blob_reset(pBlob);
     blob_set_dynamic(pBlob, zUtf8);
   }else if( useMbcs && invalid_utf8(pBlob) ){
 #if defined(_WIN32) || defined(__CYGWIN__)
