@@ -392,7 +392,9 @@ void page_dir(void){
         Blob content;
         const char *zMime = mimetype_from_name(zName);
         content_get(rid, &content);
+        safe_html_context(DOCSRC_FILE);
         wiki_render_by_mimetype(&content, zMime);
+        document_emit_js();
       }
     }
   }
@@ -912,7 +914,7 @@ void page_tree(void){
   }
   @ </ul>
   @ </ul></div>
-  style_load_one_js_file("tree.js");
+  builtin_request_js("tree.js");
   style_footer();
 
   /* We could free memory used by sTree here if we needed to.  But
