@@ -85,41 +85,49 @@ void delete_private_content(void){
 ** Usage: %fossil clone ?OPTIONS? URI FILENAME
 **
 ** Make a clone of a repository specified by URI in the local
-** file named FILENAME.
+** file named FILENAME. URI may be one of the following forms:
+** ([...] denotes optional elements):
 **
-** URI may be one of the following form: ([...] mean optional)
-**   HTTP/HTTPS protocol:
-**     http[s]://[userid[:password]@]host[:port][/path]
+**  * HTTP/HTTPS protocol:
 **
-**   SSH protocol:
-**     ssh://[userid@]host[:port]/path/to/repo.fossil\\
-**     [?fossil=path/to/fossil.exe]
+**      http[s]://[userid[:password]@]host[:port][/path]
 **
-**   Filesystem:
-**     [file://]path/to/repo.fossil
+**  * SSH protocol:
 **
-** Note 1: For ssh and filesystem, path must have an extra leading
-**         '/' to use an absolute path.
+**      ssh://[userid@]host[:port]/path/to/repo.fossil[?fossil=path/fossil.exe]
 **
-** Note 2: Use %HH escapes for special characters in the userid and
-**         password.  For example "%40" in place of "@", "%2f" in place
-**         of "/", and "%3a" in place of ":".
+**  * Filesystem:
 **
-** By default, your current login name is used to create the default
+**      [file://]path/to/repo.fossil
+**
+** Note that in Fossil (in contrast to some other DVCSes) a repository
+** is distinct from a checkout.  This command create a clone of a repository.
+** Use the separate [[open]] command to open a checkout from that repository.
+**
+** For ssh and filesystem, path must have an extra leading
+** '/' to use an absolute path.
+**
+** Use %HH escapes for special characters in the userid and
+** password.  For example "%40" in place of "@", "%2f" in place
+** of "/", and "%3a" in place of ":".
+**
+** By default, the current login name is used to create the default
 ** admin user. This can be overridden using the -A|--admin-user
 ** parameter.
 **
 ** Options:
 **    --admin-user|-A USERNAME   Make USERNAME the administrator
+**    --httpauth|-B USER:PASS    Add HTTP Basic Authorization to requests
+**    --nocompress               Omit extra delta compression
 **    --once                     Don't remember the URI.
 **    --private                  Also clone private branches
-**    --ssl-identity FILENAME    Use the SSL identity if requested by the server
+**    --save-http-password       Remember the HTTP password without asking
 **    --ssh-command|-c SSH       Use SSH as the "ssh" command
-**    --httpauth|-B USER:PASS    Add HTTP Basic Authorization to requests
+**    --ssl-identity FILENAME    Use the SSL identity if requested by the server
 **    -u|--unversioned           Also sync unversioned content
 **    -v|--verbose               Show more statistics in output
 **
-** See also: init
+** See also: [[init]], [[open]]
 */
 void clone_cmd(void){
   char *zPassword;
@@ -128,10 +136,15 @@ void clone_cmd(void){
   int nErr = 0;
   int urlFlags = URL_PROMPT_PW | URL_REMEMBER;
   int syncFlags = SYNC_CLONE;
+  int noCompress = find_option("nocompress",0,0)!=0;
 
   /* Also clone private branches */
   if( find_option("private",0,0)!=0 ) syncFlags |= SYNC_PRIVATE;
   if( find_option("once",0,0)!=0) urlFlags &= ~URL_REMEMBER;
+  if( find_option("save-http-password",0,0)!=0 ){
+    urlFlags &= ~URL_PROMPT_PW;
+    urlFlags |= URL_REMEMBER_PW;
+  }
   if( find_option("verbose","v",0)!=0) syncFlags |= SYNC_VERBOSE;
   if( find_option("unversioned","u",0)!=0 ) syncFlags |= SYNC_UNVERSIONED;
   zHttpAuth = find_option("httpauth","B",1);
@@ -156,6 +169,7 @@ void clone_cmd(void){
     file_copy(g.url.name, g.argv[3]);
     db_close(1);
     db_open_repository(g.argv[3]);
+    db_open_config(1,0);
     db_record_repository_filename(g.argv[3]);
     url_remember();
     if( !(syncFlags & SYNC_PRIVATE) ) delete_private_content();
@@ -187,19 +201,23 @@ void clone_cmd(void){
       Blob fn;
       blob_zero(&fn);
       file_canonical_name(g.zSSLIdentity, &fn, 0);
+      db_unprotect(PROTECT_ALL);
       db_set("ssl-identity", blob_str(&fn), 0);
+      db_protect_pop();
       blob_reset(&fn);
     }
+    db_unprotect(PROTECT_CONFIG);
     db_multi_exec(
       "REPLACE INTO config(name,value,mtime)"
       " VALUES('server-code', lower(hex(randomblob(20))), now());"
       "DELETE FROM config WHERE name='project-code';"
     );
+    db_protect_pop();
     url_enable_proxy(0);
     clone_ssh_db_set_options();
     url_get_password_if_needed();
     g.xlinkClusterOnly = 1;
-    nErr = client_sync(syncFlags,CONFIGSET_ALL,0);
+    nErr = client_sync(syncFlags,CONFIGSET_ALL,0,0);
     g.xlinkClusterOnly = 0;
     verify_cancel();
     db_end_transaction(0);
@@ -213,15 +231,20 @@ void clone_cmd(void){
   db_begin_transaction();
   fossil_print("Rebuilding repository meta-data...\n");
   rebuild_db(0, 1, 0);
-  fossil_print("Extra delta compression... "); fflush(stdout);
-  extra_deltification();
+  if( !noCompress ){
+    fossil_print("Extra delta compression... "); fflush(stdout);
+    extra_deltification();
+    fossil_print("\n");
+  }
   db_end_transaction(0);
-  fossil_print("\nVacuuming the database... "); fflush(stdout);
+  fossil_print("Vacuuming the database... "); fflush(stdout);
   if( db_int(0, "PRAGMA page_count")>1000
    && db_int(0, "PRAGMA page_size")<8192 ){
      db_multi_exec("PRAGMA page_size=8192;");
   }
+  db_unprotect(PROTECT_ALL);
   db_multi_exec("VACUUM");
+  db_protect_pop();
   fossil_print("\nproject-id: %s\n", db_get("project-code", 0));
   fossil_print("server-id:  %s\n", db_get("server-code", 0));
   zPassword = db_text(0, "SELECT pw FROM user WHERE login=%Q", g.zLogin);
@@ -311,7 +334,7 @@ void download_page(void){
     @ <p>Bummer.  You do not have permission to download.
     if( g.zLogin==0 || g.zLogin[0]==0 ){
       @ Maybe it would work better if you
-      @ <a href="../login">logged in</a>.
+      @ %z(href("%R/login"))logged in</a>.
     }else{
       @ Contact the site administrator and ask them to give
       @ you "Download Zip" privileges.
@@ -330,7 +353,7 @@ void download_page(void){
     @ <p>You are not authorized to clone this repository.
     if( g.zLogin==0 || g.zLogin[0]==0 ){
       @ Maybe you would be able to clone if you
-      @ <a href="../login">logged in</a>.
+      @ %z(href("%R/login"))logged in</a>.
     }else{
       @ Contact the site administrator and ask them to give
       @ you "Clone" privileges in order to clone.
