@@ -4,7 +4,7 @@
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the Simplified BSD License (also
 ** known as the "2-Clause License" or "FreeBSD License".)
-
+**
 ** This program is distributed in the hope that it will be useful,
 ** but without any warranty; without even the implied warranty of
 ** merchantability or fitness for a particular purpose.
@@ -149,7 +149,7 @@ static const struct {
   { "jpe",        3, "image/jpeg"                        },
   { "jpeg",       4, "image/jpeg"                        },
   { "jpg",        3, "image/jpeg"                        },
-  { "js",         2, "application/x-javascript"          },
+  { "js",         2, "application/javascript"            },
   { "kar",        3, "audio/midi"                        },
   { "latex",      5, "application/x-latex"               },
   { "lha",        3, "application/octet-stream"          },
@@ -192,6 +192,7 @@ static const struct {
   { "pgm",        3, "image/x-portable-graymap"          },
   { "pgn",        3, "application/x-chess-pgn"           },
   { "pgp",        3, "application/pgp"                   },
+  { "pikchr",     6, "text/x-pikchr"                     },
   { "pl",         2, "application/x-perl"                },
   { "pm",         2, "application/x-perl"                },
   { "png",        3, "image/png"                         },
@@ -400,6 +401,24 @@ static const char *mimetype_from_name_custom(const char *zSuffix){
   return 0;
 }
 
+/*
+** Emit Javascript which applies (or optionally can apply) to both the
+** /doc and /wiki pages. None of this implements required
+** functionality, just nice-to-haves. Any calls after the first are
+** no-ops.
+*/
+void document_emit_js(void){
+  static int once = 0;
+  if(0==once++){
+    builtin_fossil_js_bundle_or("pikchr", 0);
+    style_script_begin(__FILE__,__LINE__);
+    CX("window.addEventListener('load', "
+       "()=>window.fossil.pikchr.addSrcView(), "
+       "false);\n");
+    style_script_end();
+  }
+}
+
 /*
 ** Guess the mime-type of a document based on its name.
 */
@@ -752,6 +771,7 @@ void document_render(
       style_header("%s", zDefaultTitle);
       wiki_convert(pBody, 0, WIKI_BUTTONS);
     }
+    document_emit_js();
     style_footer();
   }else if( fossil_strcmp(zMime, "text/x-markdown")==0 ){
     Blob tail = BLOB_INITIALIZER;
@@ -762,18 +782,26 @@ void document_render(
       style_header("%s", zDefaultTitle);
     }
     convert_href_and_output(&tail);
+    document_emit_js();
     style_footer();
   }else if( fossil_strcmp(zMime, "text/plain")==0 ){
     style_header("%s", zDefaultTitle);
     @ <blockquote><pre>
     @ %h(blob_str(pBody))
     @ </pre></blockquote>
+    document_emit_js();
     style_footer();
   }else if( fossil_strcmp(zMime, "text/html")==0
             && doc_is_embedded_html(pBody, &title) ){
     if( blob_size(&title)==0 ) blob_append(&title,zFilename,-1);
     style_header("%s", blob_str(&title));
     convert_href_and_output(pBody);
+    document_emit_js();
+    style_footer();
+  }else if( fossil_strcmp(zMime, "text/x-pikchr")==0 ){
+    style_adunit_config(ADUNIT_RIGHT_OK);
+    style_header("%s", zDefaultTitle);
+    wiki_render_by_mimetype(pBody, zMime);
     style_footer();
 #ifdef FOSSIL_ENABLE_TH1_DOCS
   }else if( Th_AreDocsEnabled() &&
@@ -794,6 +822,7 @@ void document_render(
       Th_Render(blob_str(pBody));
     }
     if( !raw ){
+      document_emit_js();
       style_footer();
     }
 #endif
@@ -1134,25 +1163,30 @@ void background_page(void){
 /*
 ** WEBPAGE: favicon.ico
 **
-** Return the default favicon.ico image.  The returned image is for the
-** Fossil lizard icon.
+** Return the configured "favicon.ico" image.  If no "favicon.ico" image
+** is defined, the returned image is for the Fossil lizard icon.
 **
-** The intended use case here is to supply a favicon for the "fossil ui"
+** The intended use case here is to supply an icon for the "fossil ui"
 ** command.  For a permanent website, the recommended process is for
-** the admin to set up a project-specific favicon and reference that
-** icon in the HTML header using a line like:
+** the admin to set up a project-specific icon and reference that icon
+** in the HTML header using a line like:
 **
 **   <link rel="icon" href="URL-FOR-YOUR-ICON" type="MIMETYPE"/>
 ** 
 */
 void favicon_page(void){
-  Blob favicon;
+  Blob icon;
+  char *zMime;
 
   etag_check(ETAG_CONFIG, 0);
-  blob_zero(&favicon);
-  blob_init(&favicon, (char*)aLogo, sizeof(aLogo));
-  cgi_set_content_type("image/gif");
-  cgi_set_content(&favicon);
+  zMime = db_get("icon-mimetype", "image/gif");
+  blob_zero(&icon);
+  db_blob(&icon, "SELECT value FROM config WHERE name='icon-image'");
+  if( blob_size(&icon)==0 ){
+    blob_init(&icon, (char*)aLogo, sizeof(aLogo));
+  }
+  cgi_set_content_type(zMime);
+  cgi_set_content(&icon);
 }
 
 /*
