@@ -25,7 +25,7 @@
 ** every ">" into "&gt;" and every "&" into "&amp;".  Return a pointer
 ** to a new string obtained from malloc().
 **
-** We also encode " as &quot; so that it can appear as an argument
+** We also encode " as &quot; and ' as &#39; so they can appear as an argument
 ** to markup.
 */
 char *htmlize(const char *zIn, int n){
@@ -41,6 +41,7 @@ char *htmlize(const char *zIn, int n){
       case '>':   count += 4;       break;
       case '&':   count += 5;       break;
       case '"':   count += 6;       break;
+      case '\'':  count += 5;       break;
       default:    count++;          break;
     }
     i++;
@@ -76,6 +77,13 @@ char *htmlize(const char *zIn, int n){
         zOut[i++] = 't';
         zOut[i++] = ';';
         break;
+      case '\'':
+        zOut[i++] = '&';
+        zOut[i++] = '#';
+        zOut[i++] = '3';
+        zOut[i++] = '9';
+        zOut[i++] = ';';
+        break;
       default:
         zOut[i++] = c;
         break;
@@ -115,6 +123,11 @@ void htmlize_to_blob(Blob *p, const char *zIn, int n){
         blob_append(p, "&quot;", 6);
         j = i+1;
         break;
+      case '\'':
+        if( j<i ) blob_append(p, zIn+j, i-j);
+        blob_append(p, "&#39;", 5);
+        j = i+1;
+        break;
     }
   }
   if( j<i ) blob_append(p, zIn+j, i-j);
@@ -303,8 +316,9 @@ char *fossilize(const char *zIn, int nIn){
 */
 void defossilize(char *z){
   int i, j, c;
-  for(i=0; (c=z[i])!=0 && c!='\\'; i++){}
-  if( c==0 ) return;
+  char *zSlash = strchr(z, '\\');
+  if( zSlash==0 ) return;
+  i = zSlash - z;
   for(j=i; (c=z[i])!=0; i++){
     if( c=='\\' && z[i+1] ){
       i++;
@@ -326,6 +340,118 @@ void defossilize(char *z){
 }
 
 
+/*
+** The *pz variable points to a UTF8 string.  Read the next character
+** off of that string and return its codepoint value.  Advance *pz to the
+** next character
+*/
+u32 fossil_utf8_read(
+  const unsigned char **pz    /* Pointer to string from which to read char */
+){
+  unsigned int c;
+
+  /*
+  ** This lookup table is used to help decode the first byte of
+  ** a multi-byte UTF8 character.
+  */
+  static const unsigned char utf8Trans1[] = {
+    0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
+    0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
+    0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,
+    0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f,
+    0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
+    0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
+    0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
+    0x00, 0x01, 0x02, 0x03, 0x00, 0x01, 0x00, 0x00,
+  };
+
+  c = *((*pz)++);
+  if( c>=0xc0 ){
+    c = utf8Trans1[c-0xc0];
+    while( (*(*pz) & 0xc0)==0x80 ){
+      c = (c<<6) + (0x3f & *((*pz)++));
+    }
+    if( c<0x80
+        || (c&0xFFFFF800)==0xD800
+        || (c&0xFFFFFFFE)==0xFFFE ){  c = 0xFFFD; }
+  }
+  return c;
+}
+
+/*
+** Encode a UTF8 string as a JSON string literal (with or without the
+** surrounding "...", depending on whether the 2nd argument is true or
+** false) and return a pointer to the encoding.  Space to hold the
+** encoding is obtained from fossil_malloc() and must be freed by the
+** caller.
+**
+** If nOut is not NULL then it is assigned to the length, in bytes, of
+** the returned string (its strlen(), not counting the terminating
+** NUL).
+*/
+char *encode_json_string_literal(const char *zStr, int fAddQuotes,
+                                 int * nOut){
+  const unsigned char *z;
+  char *zOut;
+  u32 c;
+  int n, i, j;
+  z = (const unsigned char*)zStr;
+  n = 0;
+  while( (c = fossil_utf8_read(&z))!=0 ){
+    if( c=='\\' || c=='"' ){
+      n += 2;
+    }else if( c<' ' || c>=0x7f ){
+      if( c=='\n' || c=='\r' ){
+        n += 2;
+      }else{
+        n += 6;
+      }
+    }else{
+      n++;
+    }
+  }
+  if(fAddQuotes){
+    n += 2;
+  }
+  zOut = fossil_malloc(n+1);
+  if( zOut==0 ) return 0;
+  z = (const unsigned char*)zStr;
+  i = 0;
+  if(fAddQuotes){
+    zOut[i++] = '"';
+  }
+  while( (c = fossil_utf8_read(&z))!=0 ){
+    if( c=='\\' || c=='"' ){
+      zOut[i++] = '\\';
+      zOut[i++] = c;
+    }else if( c<' ' || c>=0x7f ){
+      zOut[i++] = '\\';
+      if( c=='\n' ){
+        zOut[i++] = 'n';
+      }else if( c=='\r' ){
+        zOut[i++] = 'r';
+      }else{
+        zOut[i++] = 'u';
+        for(j=3; j>=0; j--){
+          zOut[i+j] = "0123456789abcdef"[c&0xf];
+          c >>= 4;
+        }
+        i += 4;
+      }
+    }else{
+      zOut[i++] = c;
+    }
+  }
+  if(fAddQuotes){
+    zOut[i++] = '"';
+  }
+  zOut[i] = 0;
+  if(nOut!=0){
+    *nOut = i;
+  }
+  return zOut;
+}
+
 /*
 ** The characters used for HTTP base64 encoding.
 */
@@ -333,19 +459,12 @@ static unsigned char zBase[] =
   "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
 /*
-** Encode a string using a base-64 encoding.
-** The encoding can be reversed using the <b>decode64</b> function.
-**
-** Space to hold the result comes from malloc().
+** Translate nData bytes of content from zData into
+** ((nData+2)/3)*4) bytes of base64 encoded content and
+** put the result in z64.  Add a zero-terminator at the end.
 */
-char *encode64(const char *zData, int nData){
-  char *z64;
+int translateBase64(const char *zData, int nData, char *z64){
   int i, n;
-
-  if( nData<=0 ){
-    nData = strlen(zData);
-  }
-  z64 = fossil_malloc( (nData*4)/3 + 8 );
   for(i=n=0; i+2<nData; i+=3){
     z64[n++] = zBase[ (zData[i]>>2) & 0x3f ];
     z64[n++] = zBase[ ((zData[i]<<4) & 0x30) | ((zData[i+1]>>4) & 0x0f) ];
@@ -364,12 +483,28 @@ char *encode64(const char *zData, int nData){
     z64[n++] = '=';
   }
   z64[n] = 0;
+  return n;
+}
+
+/*
+** Encode a string using a base-64 encoding.
+** The encoding can be reversed using the <b>decode64</b> function.
+**
+** Space to hold the result comes from malloc().
+*/
+char *encode64(const char *zData, int nData){
+  char *z64;
+  if( nData<=0 ){
+    nData = strlen(zData);
+  }
+  z64 = fossil_malloc( (nData*4)/3 + 8 );
+  translateBase64(zData, nData, z64);
   return z64;
 }
 
 /*
 ** COMMAND: test-encode64
-** 
+**
 ** Usage: %fossil test-encode64 STRING
 */
 void test_encode64_cmd(void){
@@ -383,6 +518,46 @@ void test_encode64_cmd(void){
 }
 
 
+/* Decode base64 text.  Write the output into zData.  The caller
+** must ensure that zData is large enough.  It is ok for z64 and
+** zData to be the same buffer.  In other words, it is ok to decode
+** in-place.  A zero terminator is always placed at the end of zData.
+*/
+void decodeBase64(const char *z64, int *pnByte, char *zData){
+  const unsigned char *zIn = (const unsigned char*)z64;
+  int i, j, k;
+  int x[4];
+  static int isInit = 0;
+  static signed char trans[256];
+
+  if( !isInit ){
+    for(i=0; i<256; i++){ trans[i] = -1; }
+    for(i=0; zBase[i]; i++){ trans[zBase[i] & 0x7f] = i; }
+    isInit = 1;
+  }
+  for(j=k=0; zIn[0]; zIn++){
+    int v = trans[zIn[0]];
+    if( v>=0 ){
+      x[k++] = v;
+      if( k==4 ){
+        zData[j++] = ((x[0]<<2) & 0xfc) | ((x[1]>>4) & 0x03);
+        zData[j++] = ((x[1]<<4) & 0xf0) | ((x[2]>>2) & 0x0f);
+        zData[j++] = ((x[2]<<6) & 0xc0) | (x[3] & 0x3f);
+        k = 0;
+      }
+    }
+  }
+  if( k>=2 ){
+    zData[j++] = ((x[0]<<2) & 0xfc) | ((x[1]>>4) & 0x03);
+    if( k==3 ){
+      zData[j++] = ((x[1]<<4) & 0xf0) | ((x[2]>>2) & 0x0f);
+    }
+  }
+  zData[j] = 0;
+  *pnByte = j;
+}
+
+
 /*
 ** This function treats its input as a base-64 string and returns the
 ** decoded value of that string.  Characters of input that are not
@@ -394,48 +569,16 @@ void test_encode64_cmd(void){
 */
 char *decode64(const char *z64, int *pnByte){
   char *zData;
-  int n64;
-  int i, j;
-  int a, b, c, d;
-  static int isInit = 0;
-  static int trans[128];
-
-  if( !isInit ){
-    for(i=0; i<128; i++){ trans[i] = 0; }
-    for(i=0; zBase[i]; i++){ trans[zBase[i] & 0x7f] = i; }
-    isInit = 1;
-  }
-  n64 = strlen(z64);
+  int n64 = (int)strlen(z64);
   while( n64>0 && z64[n64-1]=='=' ) n64--;
   zData = fossil_malloc( (n64*3)/4 + 4 );
-  for(i=j=0; i+3<n64; i+=4){
-    a = trans[z64[i] & 0x7f];
-    b = trans[z64[i+1] & 0x7f];
-    c = trans[z64[i+2] & 0x7f];
-    d = trans[z64[i+3] & 0x7f];
-    zData[j++] = ((a<<2) & 0xfc) | ((b>>4) & 0x03);
-    zData[j++] = ((b<<4) & 0xf0) | ((c>>2) & 0x0f);
-    zData[j++] = ((c<<6) & 0xc0) | (d & 0x3f);
-  }
-  if( i+2<n64 ){
-    a = trans[z64[i] & 0x7f];
-    b = trans[z64[i+1] & 0x7f];
-    c = trans[z64[i+2] & 0x7f];
-    zData[j++] = ((a<<2) & 0xfc) | ((b>>4) & 0x03);
-    zData[j++] = ((b<<4) & 0xf0) | ((c>>2) & 0x0f);
-  }else if( i+1<n64 ){
-    a = trans[z64[i] & 0x7f];
-    b = trans[z64[i+1] & 0x7f];
-    zData[j++] = ((a<<2) & 0xfc) | ((b>>4) & 0x03);
-  }
-  zData[j] = 0;
-  *pnByte = j;
+  decodeBase64(z64, pnByte, zData);
   return zData;
 }
 
 /*
 ** COMMAND: test-decode64
-** 
+**
 ** Usage: %fossil test-decode64 STRING
 */
 void test_decode64_cmd(void){
@@ -444,7 +587,7 @@ void test_decode64_cmd(void){
   for(i=2; i<g.argc; i++){
     z = decode64(g.argv[i], &n);
     fossil_print("%d: %s\n", n, z);
-    free(z);
+    fossil_free(z);
   }
 }
 
@@ -524,6 +667,10 @@ int decode16(const unsigned char *zIn, unsigned char *pOut, int N){
 */
 int validate16(const char *zIn, int nIn){
   int i;
+  if( nIn<0 ) nIn = (int)strlen(zIn);
+  if( zIn[nIn]==0 ){
+    return strspn(zIn,"0123456789abcdefABCDEF")==nIn;
+  }
   for(i=0; i<nIn; i++, zIn++){
     if( zDecode[zIn[0]&0xff]>63 ){
       return zIn[0]==0;
@@ -544,6 +691,31 @@ void canonical16(char *z, int n){
   }
 }
 
+/*
+** Decode a string encoded using "quoted-printable".
+**
+**   (1)  "=" followed by two hex digits becomes a single
+**        byte specified by the two digits
+**
+** The decoding is done in-place.
+*/
+void decodeQuotedPrintable(char *z, int *pnByte){
+  int i, j, c;
+  for(i=j=0; (c = z[i])!=0; i++){
+    if( c=='=' ){
+      if( z[i+1]!='\r' ){
+        decode16((unsigned char*)&z[i+1], (unsigned char*)&z[j], 2);
+        j++;
+      }
+      i += 2;
+    }else{
+      z[j++] = c;
+    }
+  }
+  if( pnByte ) *pnByte = j;
+  z[j] = 0;
+}
+
 /* Randomness used for XOR-ing by the obscure() and unobscure() routines */
 static const unsigned char aObscurer[16] = {
     0xa7, 0x21, 0x31, 0xe3, 0x2a, 0x50, 0x2c, 0x86,
