@@ -28,7 +28,7 @@
 */
 struct ImportFile {
   char *zName;           /* Name of a file */
-  char *zUuid;           /* UUID of the file */
+  char *zUuid;           /* Hash of the file */
   char *zPrior;          /* Prior name if the name was changed */
   char isFrom;           /* True if obtained from the parent */
   char isExe;            /* True if executable */
@@ -61,7 +61,7 @@ static struct {
   char *zDate;                /* Date/time stamp */
   char *zUser;                /* User name */
   char *zComment;             /* Comment of a commit */
-  char *zFrom;                /* from value as a UUID */
+  char *zFrom;                /* from value as a hash */
   char *zPrevCheckin;         /* Name of the previous check-in */
   char *zFromMark;            /* The mark of the "from" field */
   int nMerge;                 /* Number of merge values */
@@ -70,6 +70,7 @@ static struct {
   int nFile;                  /* Number of aFile values */
   int nFileAlloc;             /* Number of slots in aFile[] */
   ImportFile *aFile;          /* Information about files in a commit */
+  ImportFile *pInlineFile;    /* File marked "inline" */
   int fromLoaded;             /* True zFrom content loaded into aFile[] */
   int tagCommit;              /* True if the commit adds a tag */
 } gg;
@@ -145,13 +146,14 @@ static void import_reset(int freeAll){
 ** If zMark is not zero, create a cross-reference from that mark back
 ** to the newly inserted artifact.
 **
-** If saveUuid is true, then pContent is a commit record.  Record its
-** UUID in gg.zPrevCheckin.
+** If saveHash is true, then pContent is a commit record.  Record its
+** artifact hash in gg.zPrevCheckin.
 */
 static int fast_insert_content(
   Blob *pContent,          /* Content to insert */
   const char *zMark,       /* Label using this mark, if not NULL */
-  int saveUuid,            /* Save artifact hash in gg.zPrevCheckin */
+  ImportFile *pFile,       /* Save hash on this file, if not NULL */
+  int saveHash,            /* Save artifact hash in gg.zPrevCheckin */
   int doParse              /* Invoke manifest_crosslink() */
 ){
   Blob hash;
@@ -162,8 +164,10 @@ static int fast_insert_content(
   rid = db_int(0, "SELECT rid FROM blob WHERE uuid=%B", &hash);
   if( rid==0 ){
     static Stmt ins;
+    assert( g.rcvid>0 );
     db_static_prepare(&ins,
-        "INSERT INTO blob(uuid, size, content) VALUES(:uuid, :size, :content)"
+        "INSERT INTO blob(uuid, size, rcvid, content)"
+        "VALUES(:uuid, :size, %d, :content)", g.rcvid
     );
     db_bind_text(&ins, ":uuid", blob_str(&hash));
     db_bind_int(&ins, ":size", gg.nData);
@@ -189,23 +193,38 @@ static int fast_insert_content(
         &hash, rid, &hash
     );
   }
-  if( saveUuid ){
+  if( saveHash ){
     fossil_free(gg.zPrevCheckin);
     gg.zPrevCheckin = fossil_strdup(blob_str(&hash));
   }
+  if( pFile ){
+    fossil_free(pFile->zUuid);
+    pFile->zUuid = fossil_strdup(blob_str(&hash));
+  }
   blob_reset(&hash);
   return rid;
 }
 
+/*
+** Check to ensure the file in gg.aData,gg.nData is not a control
+** artifact.  Then add the file to the repository.
+*/
+static void check_and_add_file(const char *zMark, ImportFile *pFile){
+  Blob content;
+  blob_init(&content, gg.aData, gg.nData);
+  if( gg.nData && manifest_is_well_formed(gg.aData, gg.nData) ){
+    sterilize_manifest(&content, -1);
+  }
+  fast_insert_content(&content, zMark, pFile, 0, 0);
+  blob_reset(&content);
+}
+
 /*
 ** Use data accumulated in gg from a "blob" record to add a new file
 ** to the BLOB table.
 */
 static void finish_blob(void){
-  Blob content;
-  blob_init(&content, gg.aData, gg.nData);
-  fast_insert_content(&content, gg.zMark, 0, 0);
-  blob_reset(&content);
+  check_and_add_file(gg.zMark, 0);
   import_reset(0);
 }
 
@@ -226,7 +245,7 @@ static void finish_tag(void){
     blob_appendf(&record, "\nU %F\n", gg.zUser);
     md5sum_blob(&record, &cksum);
     blob_appendf(&record, "Z %b\n", &cksum);
-    fast_insert_content(&record, 0, 0, 1);
+    fast_insert_content(&record, 0, 0, 0, 1);
     blob_reset(&cksum);
     blob_reset(&record);
   }
@@ -325,7 +344,7 @@ static void finish_commit(void){
   blob_appendf(&record, "U %F\n", gg.zUser);
   md5sum_blob(&record, &cksum);
   blob_appendf(&record, "Z %b\n", &cksum);
-  fast_insert_content(&record, gg.zMark, 1, 1);
+  fast_insert_content(&record, gg.zMark, 0, 1, 1);
   blob_reset(&cksum);
 
   /* The "git fast-export" command might output multiple "commit" lines
@@ -413,7 +432,7 @@ static char *rest_of_line(char **pzIn){
 }
 
 /*
-** Convert a "mark" or "committish" into the UUID.
+** Convert a "mark" or "committish" into the artifact hash.
 */
 static char *resolve_committish(const char *zCommittish){
   char *zRes;
@@ -573,7 +592,7 @@ static void git_fast_import(FILE *pIn){
       ** documentation.  We had to figure it out via trial and error.
       */
       for(i=5; i<strlen(zRefName) && zRefName[i]!='/'; i++){}
-      gg.tagCommit = strncmp(&zRefName[5], "tags", 4)==0;  /* True for pattern B */
+      gg.tagCommit = strncmp(&zRefName[5], "tags", 4)==0; /* pattern B */
       if( zRefName[i+1]!=0 ) zRefName += i+1;
       if( fossil_strcmp(zRefName, "master")==0 ) zRefName = ggit.zMasterName;
       gg.zBranch = fossil_strdup(zRefName);
@@ -624,6 +643,10 @@ static void git_fast_import(FILE *pIn){
           gg.nData = 0;
         }
       }
+      if( gg.pInlineFile ){
+        check_and_add_file(0, gg.pInlineFile);
+        gg.pInlineFile = 0;
+      }
     }else
     if( (!ggit.authorFlag && strncmp(zLine, "author ", 7)==0)
         || (ggit.authorFlag && strncmp(zLine, "committer ",10)==0
@@ -659,7 +682,7 @@ static void git_fast_import(FILE *pIn){
         secSince1970 = secSince1970*10 + *zTo - '0';
       }
       fossil_free(gg.zDate);
-      gg.zDate = db_text(0, "SELECT datetime(%lld, 'unixepoch')", secSince1970);
+      gg.zDate = db_text(0, "SELECT datetime(%lld, 'unixepoch')",secSince1970);
       gg.zDate[10] = 'T';
     }else
     if( strncmp(zLine, "from ", 5)==0 ){
@@ -694,7 +717,12 @@ static void git_fast_import(FILE *pIn){
       pFile->isExe = (fossil_strcmp(zPerm, "100755")==0);
       pFile->isLink = (fossil_strcmp(zPerm, "120000")==0);
       fossil_free(pFile->zUuid);
-      pFile->zUuid = resolve_committish(zUuid);
+      if( strcmp(zUuid,"inline")==0 ){
+        pFile->zUuid = 0;
+        gg.pInlineFile = pFile;
+      }else{
+        pFile->zUuid = resolve_committish(zUuid);
+      }
       pFile->isFrom = 0;
     }else
     if( strncmp(zLine, "D ", 2)==0 ){
@@ -728,7 +756,7 @@ static void git_fast_import(FILE *pIn){
         if( strlen(pFile->zName)>nFrom ){
           pNew->zName = mprintf("%s%s", zTo, pFile->zName[nFrom]);
         }else{
-          pNew->zName = fossil_strdup(pFile->zName);
+          pNew->zName = fossil_strdup(zTo);
         }
         pNew->isExe = pFile->isExe;
         pNew->isLink = pFile->isLink;
@@ -751,7 +779,7 @@ static void git_fast_import(FILE *pIn){
         if( strlen(pFile->zName)>nFrom ){
           pNew->zName = mprintf("%s%s", zTo, pFile->zName[nFrom]);
         }else{
-          pNew->zName = fossil_strdup(pFile->zName);
+          pNew->zName = fossil_strdup(zTo);
         }
         pNew->zPrior = pFile->zName;
         pNew->isExe = pFile->isExe;
@@ -762,7 +790,6 @@ static void git_fast_import(FILE *pIn){
         *pFile = *pNew;
         memset(pNew, 0, sizeof(*pNew));
       }
-      fossil_fatal("cannot handle R records, use --full-tree");
     }else
     if( strncmp(zLine, "deleteall", 9)==0 ){
       gg.fromLoaded = 1;
@@ -770,7 +797,15 @@ static void git_fast_import(FILE *pIn){
     if( strncmp(zLine, "N ", 2)==0 ){
       /* No-op */
     }else
-
+    if( strncmp(zLine, "property branch-nick ", 21)==0 ){
+      /* Breezy uses this property to store the branch name.
+      ** It has two values. Integer branch number, then the 
+      ** user-readable branch name. */
+      z = &zLine[21];
+      next_token(&z);
+      fossil_free(gg.zBranch);
+      gg.zBranch = fossil_strdup(next_token(&z));
+    }else
     {
       goto malformed_line;
     }
@@ -1351,7 +1386,8 @@ static void svn_dump_import(FILE *pIn){
   );
   db_prepare(&cpyPath,
     "INSERT INTO xfiles (tpath, tbranch, tuuid, tperm)"
-    " SELECT :path||:sep||substr(filename, length(:srcpath)+2), :branch, uuid, perm"
+    " SELECT :path||:sep||substr(filename,"
+          "  length(:srcpath)+2), :branch, uuid, perm"
     " FROM xfoci"
     " WHERE checkinID=:rid"
     "   AND filename>:srcpath||'/'"
@@ -1596,7 +1632,7 @@ static void svn_dump_import(FILE *pIn){
 }
 
 /*
-** COMMAND: import
+** COMMAND: import*
 **
 ** Usage: %fossil import ?--git? ?OPTIONS? NEW-REPOSITORY ?INPUT-FILE?
 **    or: %fossil import --svn ?OPTIONS? NEW-REPOSITORY ?INPUT-FILE?
@@ -1761,12 +1797,14 @@ void import_cmd(void){
   }
   db_open_repository(g.argv[2]);
   db_open_config(0, 0);
+  db_unprotect(PROTECT_ALL);
 
   db_begin_transaction();
   if( !incrFlag ){
     db_initial_setup(0, 0, zDefaultUser);
     db_set("main-branch", gimport.zTrunkName, 0);
   }
+  content_rcvid_init(svnFlag ? "svn-import" : "git-import");
 
   if( svnFlag ){
     db_multi_exec(
@@ -1830,9 +1868,10 @@ void import_cmd(void){
     ** the import.
     **
     ** The XMARK table provides a mapping from fast-import "marks" and symbols
-    ** into artifact ids (UUIDs - the 40-byte hex SHA1 hash of artifacts).
+    ** into artifact hashes.
+    **
     ** Given any valid fast-import symbol, the corresponding fossil rid and
-    ** uuid can found by searching against the xmark.tname field.
+    ** hash can found by searching against the xmark.tname field.
     **
     ** The XBRANCH table maps commit marks and symbols into the branch those
     ** commits belong to.  If xbranch.tname is a fast-import symbol for a
@@ -1869,7 +1908,7 @@ void import_cmd(void){
     while( db_step(&q)==SQLITE_ROW ){
       Blob record;
       db_ephemeral_blob(&q, 0, &record);
-      fast_insert_content(&record, 0, 0, 1);
+      fast_insert_content(&record, 0, 0, 0, 1);
       import_reset(0);
     }
     db_finalize(&q);
