@@ -291,8 +291,8 @@ static int after_blank_line(const char *z){
 /*
 ** Remove the PGP signature from the artifact, if there is one.
 */
-static void remove_pgp_signature(char **pz, int *pn){
-  char *z = *pz;
+static void remove_pgp_signature(const char **pz, int *pn){
+  const char *z = *pz;
   int n = *pn;
   int i;
   if( strncmp(z, "-----BEGIN PGP SIGNED MESSAGE-----", 34)!=0 ) return;
@@ -471,7 +471,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
 
   /* Strip off the PGP signature if there is one.
   */
-  remove_pgp_signature(&z, &n);
+  remove_pgp_signature((const char**)&z, &n);
 
   /* Verify that the first few characters of the artifact look like
   ** a control artifact.
@@ -483,10 +483,19 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
   }
   /* Then verify the Z-card.
   */
+#if 1
+  /* Disable this ***ONLY*** (ONLY!) when testing hand-written inputs
+     for card-related syntax errors. */
   if( verify_z_card(z, n, pErr)==2 ){
     blob_reset(pContent);
     return 0;
   }
+#else
+#warning ACHTUNG - z-card check is disabled for testing purposes.
+  if(0 && verify_z_card(NULL, 0, NULL)){
+    /*avoid unused static func error*/
+  }
+#endif
 
   /* Allocate a Manifest object to hold the parsed control artifact.
   */
@@ -603,6 +612,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         p->rEventDate = db_double(0.0,"SELECT julianday(%Q)", next_token(&x,0));
         if( p->rEventDate<=0.0 ) SYNTAX("malformed date on E-card");
         p->zEventId = next_token(&x, &sz);
+        if( p->zEventId==0 ) SYNTAX("missing hash on E-card");
         if( !hname_validate(p->zEventId, sz) ){
           SYNTAX("malformed hash on E-card");
         }
@@ -627,6 +637,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         }
         zUuid = next_token(&x, &sz);
         if( p->zBaseline==0 || zUuid!=0 ){
+          if( zUuid==0 ) SYNTAX("missing hash on F-card");
           if( !hname_validate(zUuid,sz) ){
             SYNTAX("F-card hash invalid");
           }
@@ -645,13 +656,20 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
                                     p->nFileAlloc*sizeof(p->aFile[0]) );
         }
         i = p->nFile++;
+        if( i>0 && fossil_strcmp(p->aFile[i-1].zName, zName)>=0 ){
+          SYNTAX("incorrect F-card sort order");
+        }
+        if( file_is_reserved_name(zName,-1) ){
+          /* If reserved names leaked into historical manifests due to
+          ** slack oversight by older versions of Fossil, simply ignore
+          ** those files */
+          p->nFile--;
+          break;
+        }
         p->aFile[i].zName = zName;
         p->aFile[i].zUuid = zUuid;
         p->aFile[i].zPerm = zPerm;
         p->aFile[i].zPrior = zPriorName;
-        if( i>0 && fossil_strcmp(p->aFile[i-1].zName, zName)>=0 ){
-          SYNTAX("incorrect F-card sort order");
-        }
         p->type = CFTYPE_MANIFEST;
         break;
       }
@@ -1066,7 +1084,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
 
 manifest_syntax_error:
   {
-    char *zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
+    char *zUuid = rid_to_uuid(rid);
     if( zUuid ){
       blob_appendf(pErr, "artifact [%s] ", zUuid);
       fossil_free(zUuid);
@@ -1127,6 +1145,47 @@ Manifest *manifest_get_by_name(const char *zName, int *pRid){
   return p;
 }
 
+/*
+** The input blob is text that may or may not be a valid Fossil
+** control artifact of some kind.  This routine returns true if
+** the input is a well-formed control artifact and false if it
+** is not.
+**
+** This routine is optimized to return false quickly and with minimal
+** work in the common case where the input is some random file.
+*/
+int manifest_is_well_formed(const char *zIn, int nIn){
+  int i;
+  int iRes;
+  Manifest *pManifest;
+  Blob copy, errmsg;
+  remove_pgp_signature(&zIn, &nIn);
+
+  /* Check to see that the file begins with a "card" */
+  if( nIn<3 ) return 0;
+  if( zIn[0]<'A' || zIn[0]>'M' || zIn[1]!=' ' ) return 0;
+
+  /* Check to see that the first card is followed by one more card */
+  for(i=2; i<nIn && zIn[i]!='\n'; i++){}
+  if( i>=nIn-3 ) return 0;
+  i++;
+  if( !fossil_isupper(zIn[i]) || zIn[i]<zIn[0] || zIn[i+1]!=' ' ) return 0;
+
+  /* The checks above will eliminate most random inputs.  If these
+  ** quick checks pass, then we could be dealing with a well-formed
+  ** control artifact.  Make a copy, and run it through the official
+  ** artifact parser.  This is the slow path, but it is rarely taken.
+  */
+  blob_init(&copy, 0, 0);
+  blob_init(&errmsg, 0, 0);
+  blob_append(&copy, zIn, nIn);
+  pManifest = manifest_parse(&copy, 0, &errmsg);
+  iRes = pManifest!=0;  
+  manifest_destroy(pManifest);
+  blob_reset(&errmsg);
+  return iRes;
+}
+
 /*
 ** COMMAND: test-parse-manifest
 **
@@ -1140,20 +1199,34 @@ void manifest_test_parse_cmd(void){
   Blob b;
   int i;
   int n = 1;
-  db_find_and_open_repository(0,0);
+  int isWF;
+  db_find_and_open_repository(OPEN_SUBSTITUTE|OPEN_OK_NOT_FOUND,0);
   verify_all_options();
   if( g.argc!=3 && g.argc!=4 ){
     usage("FILENAME");
   }
   blob_read_from_file(&b, g.argv[2], ExtFILE);
   if( g.argc>3 ) n = atoi(g.argv[3]);
+  isWF = manifest_is_well_formed(blob_buffer(&b), blob_size(&b));
+  fossil_print("manifest_is_well_formed() reports the input %s\n",
+       isWF ? "is ok" : "contains errors");
   for(i=0; i<n; i++){
     Blob b2;
     Blob err;
     blob_copy(&b2, &b);
     blob_zero(&err);
     p = manifest_parse(&b2, 0, &err);
-    if( p==0 ) fossil_print("ERROR: %s\n", blob_str(&err));
+    if( p==0 ){
+      fossil_print("ERROR: %s\n", blob_str(&err));
+    }else if( i==0 || (n==2 && i==1) ){
+      fossil_print("manifest_parse() worked\n");
+    }else if( i==n-1 ){
+      fossil_print("manifest_parse() worked %d more times\n", n-1);
+    }
+    if( (p==0 && isWF) || (p!=0 && !isWF) ){
+      fossil_print("ERROR: manifest_is_well_formed() and "
+                   "manifest_parse() disagree!\n");
+    }
     blob_reset(&err);
     manifest_destroy(p);
   }
@@ -1163,14 +1236,21 @@ void manifest_test_parse_cmd(void){
 /*
 ** COMMAND: test-parse-all-blobs
 **
-** Usage: %fossil test-parse-all-blobs [--limit N]
+** Usage: %fossil test-parse-all-blobs ?OPTIONS?
 **
 ** Parse all entries in the BLOB table that are believed to be non-data
 ** artifacts and report any errors.  Run this test command on historical
 ** repositories after making any changes to the manifest_parse()
 ** implementation to confirm that the changes did not break anything.
 **
-** If the --limit N argument is given, parse no more than N blobs
+** Options:
+**
+**   --limit N            Parse no more than N artifacts before stopping.
+**   --wellformed         Use all BLOB table entries as input, not just
+**                        those entries that are believed to be valid
+**                        artifacts, and verify that the result the
+**                        manifest_is_well_formed() agrees with the
+**                        result of manifest_parse().
 */
 void manifest_test_parse_all_blobs_cmd(void){
   Manifest *p;
@@ -1179,22 +1259,46 @@ void manifest_test_parse_all_blobs_cmd(void){
   int nTest = 0;
   int nErr = 0;
   int N = 1000000000;
+  int bWellFormed;
   const char *z;
   db_find_and_open_repository(0, 0);
   z = find_option("limit", 0, 1);
   if( z ) N = atoi(z);
+  bWellFormed = find_option("wellformed",0,0)!=0;
   verify_all_options();
-  db_prepare(&q, "SELECT DISTINCT objid FROM EVENT");
+  if( bWellFormed ){
+    db_prepare(&q, "SELECT rid FROM blob ORDER BY rid");
+  }else{
+    db_prepare(&q, "SELECT DISTINCT objid FROM EVENT ORDER BY objid");
+  }
   while( (N--)>0 && db_step(&q)==SQLITE_ROW ){
     int id = db_column_int(&q,0);
     fossil_print("Checking %d       \r", id);
     nTest++;
     fflush(stdout);
     blob_init(&err, 0, 0);
-    p = manifest_get(id, CFTYPE_ANY, &err);
-    if( p==0 ){
-      fossil_print("%d ERROR: %s\n", id, blob_str(&err));
-      nErr++;
+    if( bWellFormed ){
+      Blob content;
+      int isWF;
+      content_get(id, &content);
+      isWF = manifest_is_well_formed(blob_buffer(&content),blob_size(&content));
+      p = manifest_parse(&content, id, &err);
+      if( isWF && p==0 ){
+        fossil_print("%d ERROR: manifest_is_well_formed() reported true "
+                     "but manifest_parse() reports an error: %s\n",
+                     id, blob_str(&err));
+        nErr++;
+      }else if( !isWF && p!=0 ){
+        fossil_print("%d ERROR: manifest_is_well_formed() reported false "
+                     "but manifest_parse() found nothing wrong.\n", id);
+        nErr++;
+      }
+    }else{            
+      p = manifest_get(id, CFTYPE_ANY, &err);
+      if( p==0 ){
+        fossil_print("%d ERROR: %s\n", id, blob_str(&err));
+        nErr++;
+      }
     }
     blob_reset(&err);
     manifest_destroy(p);
@@ -1329,7 +1433,7 @@ static int filename_to_fnid(const char *zFilename){
 ** Compute an appropriate mlink.mperm integer for the permission string
 ** of a file.
 */
-int manifest_file_mperm(ManifestFile *pFile){
+int manifest_file_mperm(const ManifestFile *pFile){
   int mperm = PERM_REG;
   if( pFile && pFile->zPerm){
     if( strstr(pFile->zPerm,"x")!=0 ){
@@ -1702,6 +1806,13 @@ static void add_mlink(
       if( pmid<=0 ) continue;
       add_mlink(pmid, 0, mid, pChild, 0);
     }
+    for(i=0; i<pChild->nCherrypick; i++){
+      if( pChild->aCherrypick[i].zCPTarget[0]=='+'
+       && (pmid = uuid_to_rid(pChild->aCherrypick[i].zCPTarget+1, 0))>0
+      ){
+        add_mlink(pmid, 0, mid, pChild, 0);
+      }
+    }
   }
 }
 
@@ -1724,6 +1835,7 @@ static int manifest_add_checkin_linkages(
   int parentid = 0;
   char zBaseId[30];    /* Baseline manifest RID for deltas.  "NULL" otherwise */
   Stmt q;
+  int nLink;
 
   if( p->zBaseline ){
      sqlite3_snprintf(sizeof(zBaseId), zBaseId, "%d",
@@ -1740,7 +1852,11 @@ static int manifest_add_checkin_linkages(
     if( i==0 ) parentid = pid;
   }
   add_mlink(parentid, 0, rid, p, 1);
-  if( nParent>1 ){
+  nLink = nParent;
+  for(i=0; i<p->nCherrypick; i++){
+    if( p->aCherrypick[i].zCPTarget[0]=='+' ) nLink++;
+  }
+  if( nLink>1 ){
     /* Change MLINK.PID from 0 to -1 for files that are added by merge. */
     db_multi_exec(
       "UPDATE mlink SET pid=-1"
@@ -1749,7 +1865,7 @@ static int manifest_add_checkin_linkages(
       "   AND fnid IN "
       "  (SELECT fnid FROM mlink WHERE mid=%d GROUP BY fnid"
       "    HAVING count(*)<%d)",
-      rid, rid, nParent
+      rid, rid, nLink
     );
   }
   db_prepare(&q, "SELECT cid, isprim FROM plink WHERE pid=%d", rid);
@@ -1798,11 +1914,7 @@ void manifest_reparent_checkin(int rid, const char *zValue){
     z[j] = 0;
     i += j;
   }
-  if( !db_exists("SELECT 1 FROM plink WHERE cid=%d AND pid=%d",
-                 rid, uuid_to_rid(azParent[0],0))
-  ){
-    p = manifest_get(rid, CFTYPE_MANIFEST, 0);
-  }
+  p = manifest_get(rid, CFTYPE_MANIFEST, 0);
   if( p!=0 ){
     db_multi_exec(
        "DELETE FROM plink WHERE cid=%d;"
@@ -1810,8 +1922,8 @@ void manifest_reparent_checkin(int rid, const char *zValue){
        rid, rid
     );
     manifest_add_checkin_linkages(rid,p,nParent,azParent);
+    manifest_destroy(p);
   }
-  manifest_destroy(p);
 reparent_abort:
   fossil_free(azParent);
   fossil_free(zCopy);
@@ -2040,17 +2152,18 @@ void manifest_ticket_event(
 ** However, if the manifest is PGP signed then the extra line has to be
 ** inserted before the PGP signature (thus invalidating the signature).
 */
-void sterilize_manifest(Blob *p){
+void sterilize_manifest(Blob *p, int eType){
   char *z, *zOrig;
   int n, nOrig;
   static const char zExtraLine[] =
-      "# Remove this line to create a well-formed manifest.\n";
+      "# Remove this line to create a well-formed Fossil %s.\n";
+  const char *zType = eType==CFTYPE_MANIFEST ? "manifest" : "control artifact";
 
   z = zOrig = blob_materialize(p);
   n = nOrig = blob_size(p);
-  remove_pgp_signature(&z, &n);
+  remove_pgp_signature((const char **)&z, &n);
   if( z==zOrig ){
-    blob_append(p, zExtraLine, -1);
+    blob_appendf(p, zExtraLine/*works-like:"%s"*/, zType);
   }else{
     int iEnd;
     Blob copy;
@@ -2058,7 +2171,7 @@ void sterilize_manifest(Blob *p){
     blob_init(p, 0, 0);
     iEnd = (int)(&z[n] - zOrig);
     blob_append(p, zOrig, iEnd);
-    blob_append(p, zExtraLine, -1);
+    blob_appendf(p, zExtraLine/*works-like:"%s"*/, zType);
     blob_append(p, &zOrig[iEnd], -1);
     blob_zero(&copy);
   }
@@ -2436,7 +2549,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     ** process the attachment artifact before the artifact to
     ** which it is attached!) */
     char attachToType = 'w';
-    if( fossil_is_uuid(p->zAttachTarget) ){
+    if( fossil_is_artifact_hash(p->zAttachTarget) ){
       if( db_exists("SELECT 1 FROM tag WHERE tagname='tkt-%q'",
             p->zAttachTarget)
         ){
@@ -2664,6 +2777,9 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
       );
       fossil_free(zTitle);
     }
+    if( p->zWiki[0] ){
+      backlink_extract(p->zWiki, p->zMimetype, rid, BKLNK_FORUM, p->rDate, 1);
+    }
   }
   db_end_transaction(0);
   if( permitHooks ){
