@@ -31,10 +31,19 @@ void markdown_to_html(
 
 #endif /* INTERFACE */
 
+/*
+** An instance of the following structure is passed through the
+** "opaque" pointer.
+*/
+typedef struct MarkdownToHtml MarkdownToHtml;
+struct MarkdownToHtml {
+  Blob *output_title;     /* Store the title here */
+};
+
 
 /* INTER_BLOCK -- skip a line between block level elements */
 #define INTER_BLOCK(ob) \
-  do { if( blob_size(ob)>0 ) blob_append(ob, "\n", 1); } while (0)
+  do { if( blob_size(ob)>0 ) blob_append_char(ob, '\n'); } while (0)
 
 /* BLOB_APPEND_LITERAL -- append a string literal to a blob */
 #define BLOB_APPEND_LITERAL(blob, literal) \
@@ -50,9 +59,12 @@ void markdown_to_html(
   blob_append((dest), blob_buffer(src), blob_size(src))
 
 
-/* HTML escape */
-
-static void html_escape(struct Blob *ob, const char *data, size_t size){
+/* HTML escapes
+**
+** html_escape() converts < to &lt;, > to &gt;, and & to &amp;.
+** html_quote() goes further and converts " into &quot; and ' in &#39;.
+*/
+static void html_quote(struct Blob *ob, const char *data, size_t size){
   size_t beg = 0, i = 0;
   while( i<size ){
     beg = i;
@@ -61,6 +73,7 @@ static void html_escape(struct Blob *ob, const char *data, size_t size){
      && data[i]!='>'
      && data[i]!='"'
      && data[i]!='&'
+     && data[i]!='\''
     ){
       i++;
     }
@@ -74,6 +87,34 @@ static void html_escape(struct Blob *ob, const char *data, size_t size){
         BLOB_APPEND_LITERAL(ob, "&amp;");
       }else if( data[i]=='"' ){
         BLOB_APPEND_LITERAL(ob, "&quot;");
+      }else if( data[i]=='\'' ){
+        BLOB_APPEND_LITERAL(ob, "&#39;");
+      }else{
+        break;
+      }
+      i++;
+    }
+  }
+}
+static void html_escape(struct Blob *ob, const char *data, size_t size){
+  size_t beg = 0, i = 0;
+  while( i<size ){
+    beg = i;
+    while( i<size
+     && data[i]!='<'
+     && data[i]!='>'
+     && data[i]!='&'
+    ){
+      i++;
+    }
+    blob_append(ob, data+beg, i-beg);
+    while( i<size ){
+      if( data[i]=='<' ){
+        BLOB_APPEND_LITERAL(ob, "&lt;");
+      }else if( data[i]=='>' ){
+        BLOB_APPEND_LITERAL(ob, "&gt;");
+      }else if( data[i]=='&' ){
+        BLOB_APPEND_LITERAL(ob, "&amp;");
       }else{
         break;
       }
@@ -99,10 +140,10 @@ static void html_epilog(struct Blob *ob, void *opaque){
   BLOB_APPEND_LITERAL(ob, "</div>\n");
 }
 
-static void html_raw_block(struct Blob *ob, struct Blob *text, void *opaque){
+static void html_blockhtml(struct Blob *ob, struct Blob *text, void *opaque){
   char *data = blob_buffer(text);
   size_t size = blob_size(text);
-  Blob *title = (Blob*)opaque;
+  Blob *title = ((MarkdownToHtml*)opaque)->output_title;
   while( size>0 && fossil_isspace(data[0]) ){ data++; size--; }
   while( size>0 && fossil_isspace(data[size-1]) ){ size--; }
   /* If the first raw block is an <h1> element, then use it as the title. */
@@ -112,7 +153,7 @@ static void html_raw_block(struct Blob *ob, struct Blob *text, void *opaque){
    && sqlite3_strnicmp("<h1",data,3)==0
    && sqlite3_strnicmp("</h1>", &data[size-5],5)==0
   ){
-    int nTag = htmlTagLength(data);
+    int nTag = html_tag_length(data);
     blob_append(title, data+nTag, size - nTag - 5);
     return;
   }
@@ -141,7 +182,7 @@ static void html_header(
   int level,
   void *opaque
 ){
-  struct Blob *title = opaque;
+  struct Blob *title = ((MarkdownToHtml*)opaque)->output_title;
   /* The first header at the beginning of a text is considered as
    * a title and not output. */
   if( blob_size(ob)<=PROLOG_SIZE && title!=0 && blob_size(title)==0 ){
@@ -268,9 +309,8 @@ static void html_table_row(
 
 /* HTML span tags */
 
-static int html_raw_span(struct Blob *ob, struct Blob *text, void *opaque){
-  /* If the document begins with a <h1> markup, take that as the header. */
-  BLOB_APPEND_BLOB(ob, text);
+static int html_raw_html_tag(struct Blob *ob, struct Blob *text, void *opaque){
+  blob_append(ob, blob_buffer(text), blob_size(text));
   return 1;
 }
 
@@ -283,7 +323,7 @@ static int html_autolink(
   if( !link || blob_size(link)<=0 ) return 0;
   BLOB_APPEND_LITERAL(ob, "<a href=\"");
   if( type==MKDA_IMPLICIT_EMAIL ) BLOB_APPEND_LITERAL(ob, "mailto:");
-  html_escape(ob, blob_buffer(link), blob_size(link));
+  html_quote(ob, blob_buffer(link), blob_size(link));
   BLOB_APPEND_LITERAL(ob, "\">");
   if( type==MKDA_EXPLICIT_EMAIL && blob_size(link)>7 ){
     /* remove "mailto:" from displayed text */
@@ -295,10 +335,97 @@ static int html_autolink(
   return 1;
 }
 
-static int html_code_span(struct Blob *ob, struct Blob *text, void *opaque){
-  BLOB_APPEND_LITERAL(ob, "<code>");
-  html_escape(ob, blob_buffer(text), blob_size(text));
-  BLOB_APPEND_LITERAL(ob, "</code>");
+/*
+** The nSrc bytes at zSrc[] are Pikchr input text (allegedly).  Process that
+** text and insert the result in place of the original.
+*/
+void pikchr_to_html(
+  Blob *ob,                     /* Write the generated SVG here */
+  const char *zSrc, int nSrc,   /* The Pikchr source text */
+  const char *zArg, int nArg    /* Addition arguments */
+){
+  int pikFlags = PIKCHR_PROCESS_NONCE
+    | PIKCHR_PROCESS_DIV
+    | PIKCHR_PROCESS_SRC
+    | PIKCHR_PROCESS_ERR_PRE;
+  Blob bSrc = empty_blob;
+
+  while( nArg>0 ){
+    int i;
+    for(i=0; i<nArg && !fossil_isspace(zArg[i]); i++){}
+    if( i==6 && strncmp(zArg, "center", 6)==0 ){
+      pikFlags |= PIKCHR_PROCESS_DIV_CENTER;
+    }else if( i==6 && strncmp(zArg, "indent", 6)==0 ){
+      pikFlags |= PIKCHR_PROCESS_DIV_INDENT;
+    }else if( i==10 && strncmp(zArg, "float-left", 10)==0 ){
+      pikFlags |= PIKCHR_PROCESS_DIV_FLOAT_LEFT;
+    }else if( i==11 && strncmp(zArg, "float-right", 11)==0 ){
+      pikFlags |= PIKCHR_PROCESS_DIV_FLOAT_RIGHT;
+    }else if( i==6 && strncmp(zArg, "toggle", 6)==0 ){
+      pikFlags |= PIKCHR_PROCESS_DIV_TOGGLE;
+    }else if( i==6 && strncmp(zArg, "source", 6)==0 ){
+      pikFlags |= PIKCHR_PROCESS_DIV_SOURCE;
+    }else if( i==13 && strncmp(zArg, "source-inline", 13)==0 ){
+      pikFlags |= PIKCHR_PROCESS_DIV_SOURCE_INLINE;
+    }
+    while( i<nArg && fossil_isspace(zArg[i]) ){ i++; }
+    zArg += i;
+    nArg -= i;
+  }
+  blob_append(&bSrc, zSrc, nSrc)
+    /*have to dup input to ensure a NUL-terminated source string */;
+  pikchr_process(blob_str(&bSrc), pikFlags, 0, ob);
+  blob_reset(&bSrc);
+}
+
+/* Invoked for `...` blocks where there are nSep grave accents in a
+** row that serve as the delimiter.  According to CommonMark:
+**
+**   *  https://spec.commonmark.org/0.29/#fenced-code-blocks
+**   *  https://spec.commonmark.org/0.29/#code-spans
+**
+** If nSep is 1 or 2, then this is a code-span which is inline.
+** If nSep is 3 or more, then this is a fenced code block
+*/
+static int html_codespan(
+  struct Blob *ob,    /* Write the output here */
+  struct Blob *text,  /* The stuff in between the code span marks */
+  int nSep,           /* Number of grave accents marks as delimiters */
+  void *opaque
+){
+  if( text==0 ){
+    /* no-op */
+  }else if( nSep<=2 ){
+    /* One or two graves: an in-line code span */
+    BLOB_APPEND_LITERAL(ob, "<code>");
+    html_escape(ob, blob_buffer(text), blob_size(text));
+    BLOB_APPEND_LITERAL(ob, "</code>");
+  }else{
+    /* Three or more graves: a fenced code block */
+    int n = blob_size(text);
+    const char *z = blob_buffer(text);
+    int i;
+    for(i=0; i<n && z[i]!='\n'; i++){}
+    if( i>=n ){
+      blob_appendf(ob, "<pre><code>%#h</code></pre>", n, z);
+    }else{
+      int k, j;
+      i++;
+      for(k=0; k<i && fossil_isspace(z[k]); k++){}
+      if( k==i ){
+        blob_appendf(ob, "<pre><code>%#h</code></pre>", n-i, z+i);
+      }else{
+        for(j=k+1; j<i && !fossil_isspace(z[j]); j++){}
+        if( j-k==6 && strncmp(z+k,"pikchr",6)==0 ){
+          while( j<i && fossil_isspace(z[j]) ){ j++; }
+          pikchr_to_html(ob, z+i, n-i, z+j, i-j);
+        }else{
+          blob_appendf(ob, "<pre><code class='language-%#h'>%#h</code></pre>",
+                            j-k, z+k, n-i, z+i);
+        }
+      }
+    }
+  }
   return 1;
 }
 
@@ -334,18 +461,18 @@ static int html_image(
   void *opaque
 ){
   BLOB_APPEND_LITERAL(ob, "<img src=\"");
-  html_escape(ob, blob_buffer(link), blob_size(link));
+  html_quote(ob, blob_buffer(link), blob_size(link));
   BLOB_APPEND_LITERAL(ob, "\" alt=\"");
-  html_escape(ob, blob_buffer(alt), blob_size(alt));
+  html_quote(ob, blob_buffer(alt), blob_size(alt));
   if( title && blob_size(title)>0 ){
     BLOB_APPEND_LITERAL(ob, "\" title=\"");
-    html_escape(ob, blob_buffer(title), blob_size(title));
+    html_quote(ob, blob_buffer(title), blob_size(title));
   }
   BLOB_APPEND_LITERAL(ob, "\" />");
   return 1;
 }
 
-static int html_line_break(struct Blob *ob, void *opaque){
+static int html_linebreak(struct Blob *ob, void *opaque){
   BLOB_APPEND_LITERAL(ob, "<br />\n");
   return 1;
 }
@@ -358,20 +485,24 @@ static int html_link(
   void *opaque
 ){
   char *zLink = blob_buffer(link);
-  BLOB_APPEND_LITERAL(ob, "<a href=\"");
-  if( zLink[0]=='/' ){
-    /* For any hyperlink that begins with "/", make it refer to the root
-    ** of the Fossil repository */
-    blob_append(ob, g.zTop, -1);
+  char *zTitle = title!=0 && blob_size(title)>0 ? blob_str(title) : 0;
+  char zClose[20];
+
+  if( zLink==0 || zLink[0]==0 ){
+    zClose[0] = 0;
+  }else{  
+    static const int flags = 
+       WIKI_NOBADLINKS |
+       WIKI_MARKDOWNLINKS
+    ;
+    wiki_resolve_hyperlink(ob, flags, zLink, zClose, sizeof(zClose), 0, zTitle);
   }
-  html_escape(ob, blob_buffer(link), blob_size(link));
-  if( title && blob_size(title)>0 ){
-    BLOB_APPEND_LITERAL(ob, "\" title=\"");
-    html_escape(ob, blob_buffer(title), blob_size(title));
+  if( blob_size(content)==0 ){
+    if( link ) BLOB_APPEND_BLOB(ob, link);
+  }else{
+    BLOB_APPEND_BLOB(ob, content);
   }
-  BLOB_APPEND_LITERAL(ob, "\">");
-  BLOB_APPEND_BLOB(ob, content);
-  BLOB_APPEND_LITERAL(ob, "</a>");
+  blob_append(ob, zClose, -1);
   return 1;
 }
 
@@ -411,7 +542,7 @@ void markdown_to_html(
     /* block level elements */
     html_blockcode,
     html_blockquote,
-    html_raw_block,
+    html_blockhtml,
     html_header,
     html_hrule,
     html_list,
@@ -423,25 +554,27 @@ void markdown_to_html(
 
     /* span level elements */
     html_autolink,
-    html_code_span,
+    html_codespan,
     html_double_emphasis,
     html_emphasis,
     html_image,
-    html_line_break,
+    html_linebreak,
     html_link,
-    html_raw_span,
+    html_raw_html_tag,
     html_triple_emphasis,
 
     /* low level elements */
-    0,  /* entities are copied verbatim */
+    0,    /* entity */
     html_normal_text,
 
     /* misc. parameters */
-    64, /* maximum stack */
-    "*_", /* emphasis characters */
-    0 /* opaque data */
+    "*_", /* emph_chars */
+    0     /* opaque */
   };
-  html_renderer.opaque = output_title;
+  MarkdownToHtml context;
+  memset(&context, 0, sizeof(context));
+  context.output_title = output_title;
+  html_renderer.opaque = &context;
   if( output_title ) blob_reset(output_title);
   blob_reset(output_body);
   markdown(output_body, input_markdown, &html_renderer);
