@@ -382,7 +382,7 @@ int symbolic_name_to_rid(const char *zTag, const char *zType){
     }else{
       db_prepare(&q,
         "SELECT blob.rid"
-        "  FROM blob, event"
+        "  FROM blob CROSS JOIN event"
         " WHERE blob.uuid GLOB '%q*'"
         "   AND event.objid=blob.rid"
         "   AND event.type GLOB '%q'",
@@ -448,13 +448,19 @@ int symbolic_name_to_rid(const char *zTag, const char *zType){
 }
 
 /*
-** This routine takes a user-entered UUID which might be in mixed
-** case and might only be a prefix of the full UUID and converts it
-** into the full-length UUID in canonical form.
+** This routine takes a user-entered string and tries to convert it to
+** an artifact hash.
 **
-** If the input is not a UUID or a UUID prefix, then try to resolve
+** We first try to treat the string as an artifact hash, or at least a
+** unique prefix of an artifact hash. The input may be in mixed case.
+** If we are passed such a string, this routine has the effect of
+** converting the hash [prefix] to canonical form.
+**
+** If the input is not a hash or a hash prefix, then try to resolve
 ** the name as a tag.  If multiple tags match, pick the latest.
-** If the input name matches "tag:*" then always resolve as a tag.
+** A caller can force this routine to skip the hash case above by
+** prefixing the string with "tag:", a useful property when the tag
+** may be misinterpreted as a hex ASCII string. (e.g. "decade" or "facade")
 **
 ** If the input is not a tag, then try to match it as an ISO-8601 date
 ** string YYYY-MM-DD HH:MM:SS and pick the nearest check-in to that date.
@@ -483,12 +489,12 @@ int name_to_uuid(Blob *pName, int iErrPriority, const char *zType){
 /*
 ** This routine is similar to name_to_uuid() except in the form it
 ** takes its parameters and returns its value, and in that it does not
-** treat errors as fatal. zName must be a UUID, as described for
-** name_to_uuid(). zType is also as described for that function. If
+** treat errors as fatal. zName must be an artifact hash or prefix of
+** a hash. zType is also as described for name_to_uuid(). If
 ** zName does not resolve, 0 is returned. If it is ambiguous, a
 ** negative value is returned. On success the rid is returned and
 ** pUuid (if it is not NULL) is set to a newly-allocated string,
-** the full UUID, which must eventually be free()d by the caller.
+** the full hash, which must eventually be free()d by the caller.
 */
 int name_to_uuid2(const char *zName, const char *zType, char **pUuid){
   int rid = symbolic_name_to_rid(zName, zType);
@@ -501,8 +507,9 @@ int name_to_uuid2(const char *zName, const char *zType, char **pUuid){
 
 /*
 ** name_collisions searches through events, blobs, and tickets for
-** collisions of a given UUID based on its length on UUIDs no shorter
-** than 4 characters in length.
+** collisions of a given hash based on its length, counting only
+** hashes greater than or equal to 4 hex ASCII characters (16 bits)
+** in length.
 */
 int name_collisions(const char *zName){
   int c = 0;         /* count of collisions for zName */
@@ -594,18 +601,21 @@ int name_to_rid(const char *zName){
 **
 ** The NAME given by the name parameter is ambiguous.  Display a page
 ** that shows all possible choices and let the user select between them.
+**
+** The src= query parameter is optional.  If omitted it defaults
+** to "info".
 */
 void ambiguous_page(void){
   Stmt q;
   const char *zName = P("name");
-  const char *zSrc = P("src");
+  const char *zSrc = PD("src","info");
   char *z;
 
   if( zName==0 || zName[0]==0 || zSrc==0 || zSrc[0]==0 ){
     fossil_redirect_home();
   }
   style_header("Ambiguous Artifact ID");
-  @ <p>The artifact id <b>%h(zName)</b> is ambiguous and might
+  @ <p>The artifact hash prefix <b>%h(zName)</b> is ambiguous and might
   @ mean any of the following:
   @ <ol>
   z = mprintf("%s", zName);
@@ -635,7 +645,7 @@ void ambiguous_page(void){
     @ %s(zUuid)</a> -
     @ <ul></ul>
     @ Ticket
-    hyperlink_to_uuid(zUuid);
+    hyperlink_to_version(zUuid);
     @ - %h(zTitle).
     @ <ul><li>
     object_description(rid, 0, 0, 0);
@@ -680,7 +690,7 @@ int name_to_rid_www(const char *zParamName){
   if( zName==0 || zName[0]==0 ) return 0;
   rid = symbolic_name_to_rid(zName, "*");
   if( rid<0 ){
-    cgi_redirectf("%s/ambiguous/%T?src=%t", g.zTop, zName, g.zPath);
+    cgi_redirectf("%R/ambiguous/%T?src=%t", zName, g.zPath);
     rid = 0;
   }
   return rid;
@@ -768,6 +778,7 @@ void whatis_rid(int rid, int verboseFlag){
                  db_column_text(&q, 1));
     fossil_print("comment:    ");
     comment_print(db_column_text(&q,3), 0, 12, -1, get_comment_format());
+    cnt++;
   }
   db_finalize(&q);
 
@@ -790,6 +801,7 @@ void whatis_rid(int rid, int verboseFlag){
       db_column_text(&q, 2));
     fossil_print("            ");
     comment_print(db_column_text(&q,4), 0, 12, -1, get_comment_format());
+    cnt++;
   }
   db_finalize(&q);
 
@@ -825,8 +837,22 @@ void whatis_rid(int rid, int verboseFlag){
                  db_column_text(&q,2), db_column_text(&q,3));
     fossil_print("            ");
     comment_print(db_column_text(&q,1), 0, 12, -1, get_comment_format());
+    cnt++;
   }
   db_finalize(&q);
+
+  /* If other information available, try to describe the object */
+  if( cnt==0 ){
+    char *zWhere = mprintf("=%d", rid);
+    char *zDesc;
+    describe_artifacts(zWhere);
+    free(zWhere);
+    zDesc = db_text(0,
+       "SELECT printf('%%-12s%%s %%s',type||':',summary,substr(ref,1,16))"
+       "  FROM description WHERE rid=%d", rid);
+    fossil_print("%s\n", zDesc);
+    fossil_free(zDesc);
+  }
 }
 
 /*
@@ -968,7 +994,8 @@ static const char zDescTab[] =
 @   summary TEXT,                  -- Summary comment for the object
 @   ref TEXT                       -- hash of an object to link against
 @ );
-@ CREATE INDEX desctype ON description(summary) WHERE summary='unknown';
+@ CREATE INDEX IF NOT EXISTS desctype
+@   ON description(summary) WHERE summary='unknown';
 ;
 
 /*
@@ -1032,7 +1059,8 @@ static void describe_unknown_artifacts(){
     "   WHERE description.summary='unknown'\n"
     "     AND tagxref.tagid=(SELECT tagid FROM tag WHERE tagname='cluster')\n"
     "     AND blob.rid=tagxref.rid\n"
-    "     AND content(blob.uuid) GLOB ('*M '||blob.uuid||'*');"
+    "     AND CAST(content(blob.uuid) AS text)"
+    "                   GLOB ('*M '||description.uuid||'*');"
   );
 }
 
@@ -1048,11 +1076,15 @@ void describe_artifacts(const char *zWhere){
   db_multi_exec(
     "INSERT OR IGNORE INTO description(rid,uuid,rcvid,ctime,type,summary)\n"
     "SELECT blob.rid, blob.uuid, blob.rcvid, event.mtime, 'checkin',\n"
-    "       'check-in on ' || strftime('%%Y-%%m-%%d %%H:%%M',event.mtime)\n"
+          " 'check-in to '\n"
+          " ||  coalesce((SELECT value FROM tagxref WHERE tagid=%d"
+                     "   AND tagtype>0 AND tagxref.rid=blob.rid),'trunk')\n"
+          " || ' by ' || coalesce(event.euser,event.user)\n"
+          " || ' on ' || strftime('%%Y-%%m-%%d %%H:%%M',event.mtime)\n"
     "  FROM event, blob\n"
     " WHERE (event.objid %s) AND event.type='ci'\n"
     "   AND event.objid=blob.rid;",
-    zWhere /*safe-for-%s*/
+    TAG_BRANCH, zWhere /*safe-for-%s*/
   );
 
   /* Describe files */
@@ -1205,7 +1237,7 @@ int describe_artifacts_to_stdout(const char *zWhere, const char *zLabel){
   int cnt = 0;
   if( zWhere!=0 ) describe_artifacts(zWhere);
   db_prepare(&q,
-    "SELECT uuid, summary, isPrivate\n"
+    "SELECT uuid, summary, coalesce(ref,''), isPrivate\n"
     "  FROM description\n"
     " ORDER BY ctime, type;"
   );
@@ -1214,8 +1246,9 @@ int describe_artifacts_to_stdout(const char *zWhere, const char *zLabel){
       fossil_print("%s\n", zLabel);
       zLabel = 0;
     }
-    fossil_print("  %.16s %s", db_column_text(&q,0), db_column_text(&q,1));
-    if( db_column_int(&q,2) ) fossil_print(" (private)");
+    fossil_print("  %.16s %s %s", db_column_text(&q,0),
+           db_column_text(&q,1), db_column_text(&q,2));
+    if( db_column_int(&q,3) ) fossil_print(" (private)");
     fossil_print("\n");
     cnt++;
   }
@@ -1607,7 +1640,7 @@ static void collision_report(const char *zSql){
     for(j=0; j<aCollide[i].cnt && j<MAX_COLLIDE; j++){
       char *zId = aCollide[i].azHit[j];
       if( zId==0 ) continue;
-      @ %z(href("%R/whatis/%s",zId))%h(zId)</a>
+      @ %z(href("%R/ambiguous/%s",zId))%h(zId)</a>
     }
   }
   for(i=4; i<count(aCollide); i++){
