@@ -195,8 +195,16 @@ static int report_query_authorizer(
          "tag",
          "tagxref",
          "unversioned",
+         "backlink",
       };
       int i;
+      if( zArg1==0 ){
+        /* Some legacy versions of SQLite will sometimes send spurious
+        ** READ authorizations that have no table name.  These can be
+        ** ignored. */
+        rc = SQLITE_IGNORE;
+        break;
+      }
       if( fossil_strncmp(zArg1, "fx_", 3)==0 ){
         break;
       }
@@ -224,11 +232,11 @@ static int report_query_authorizer(
 ** Activate the query authorizer
 */
 void report_restrict_sql(char **pzErr){
-  sqlite3_set_authorizer(g.db, report_query_authorizer, (void*)pzErr);
+  db_set_authorizer(report_query_authorizer,(void*)pzErr,"Ticket-Report");
   sqlite3_limit(g.db, SQLITE_LIMIT_VDBE_OP, 10000);
 }
 void report_unrestrict_sql(void){
-  sqlite3_set_authorizer(g.db, 0, 0);
+  db_clear_authorizer();
 }
 
 
@@ -505,9 +513,9 @@ void view_edit(void){
 */
 static void report_format_hints(void){
   char *zSchema;
-  zSchema = db_text(0,"SELECT sql FROM sqlite_master WHERE name='ticket'");
+  zSchema = db_text(0,"SELECT sql FROM sqlite_schema WHERE name='ticket'");
   if( zSchema==0 ){
-    zSchema = db_text(0,"SELECT sql FROM repository.sqlite_master"
+    zSchema = db_text(0,"SELECT sql FROM repository.sqlite_schema"
                         " WHERE name='ticket'");
   }
   @ <hr /><h3>TICKET Schema</h3>
@@ -665,7 +673,7 @@ static int generate_html(
 ){
   struct GenerateHTML *pState = (struct GenerateHTML*)pUser;
   int i;
-  const char *zTid;  /* Ticket UUID.  (value of column named '#') */
+  const char *zTid;  /* Ticket hash.  (value of column named '#') */
   const char *zBg = 0; /* Use this background color */
 
   /* Do initialization
@@ -674,7 +682,7 @@ static int generate_html(
     /* Turn off the authorizer.  It is no longer doing anything since the
     ** query has already been prepared.
     */
-    sqlite3_set_authorizer(g.db, 0, 0);
+    db_clear_authorizer();
 
     /* Figure out the number of columns, the column that determines background
     ** color, and whether or not this row of data is represented by multiple
@@ -948,191 +956,6 @@ static int db_exec_readonly(
   return rc;
 }
 
-/*
-** Output Javascript code that will enables sorting of the table with
-** the id zTableId by clicking.
-**
-** The javascript was originally derived from:
-**
-**     http://www.webtoolkit.info/sortable-html-table.html
-**
-** But there have been extensive modifications.
-**
-** This variation allows column types to be expressed using the second
-** argument.  Each character of the second argument represent a column.
-**
-**       t      Sort by text
-**       n      Sort numerically
-**       k      Sort by the data-sortkey property
-**       x      This column is not sortable
-**
-** Capital letters mean sort in reverse order.
-** If there are fewer characters in zColumnTypes[] than their are columns,
-** then all extra columns assume type "t" (text).
-**
-** The third parameter is the column that was initially sorted (using 1-based
-** column numbers, like SQL).  Make this value 0 if none of the columns are
-** initially sorted.  Make the value negative if the column is initially sorted
-** in reverse order.
-**
-** Clicking on the same column header twice in a row inverts the sort.
-*/
-void output_table_sorting_javascript(
-  const char *zTableId,      /* ID of table to sort */
-  const char *zColumnTypes,  /* String for column types */
-  int iInitSort              /* Initially sorted column. Leftmost is 1. 0 for NONE */
-){
-  @ <script>
-  @ function SortableTable(tableEl,columnTypes,initSort){
-  @   this.tbody = tableEl.getElementsByTagName('tbody');
-  @   this.columnTypes = columnTypes;
-  @   var ncols = tableEl.rows[0].cells.length;
-  @   for(var i = columnTypes.length; i<=ncols; i++){this.columnTypes += 't';}
-  @   this.sort = function (cell) {
-  @     var column = cell.cellIndex;
-  @     var sortFn;
-  @     switch( cell.sortType ){
-  if( strchr(zColumnTypes,'n') ){
-    @       case "n": sortFn = this.sortNumeric;  break;
-  }
-  if( strchr(zColumnTypes,'N') ){
-    @       case "N": sortFn = this.sortReverseNumeric;  break;
-  }
-  @       case "t": sortFn = this.sortText;  break;
-  if( strchr(zColumnTypes,'T') ){
-    @       case "T": sortFn = this.sortReverseText;  break;
-  }
-  if( strchr(zColumnTypes,'k') ){
-    @       case "k": sortFn = this.sortKey;  break;
-  }
-  if( strchr(zColumnTypes,'K') ){
-    @       case "K": sortFn = this.sortReverseKey;  break;
-  }
-  @       default:  return;
-  @     }
-  @     this.sortIndex = column;
-  @     var newRows = new Array();
-  @     for (j = 0; j < this.tbody[0].rows.length; j++) {
-  @        newRows[j] = this.tbody[0].rows[j];
-  @     }
-  @     if( this.sortIndex==Math.abs(this.prevColumn)-1 ){
-  @       newRows.reverse();
-  @       this.prevColumn = -this.prevColumn;
-  @     }else{
-  @       newRows.sort(sortFn);
-  @       this.prevColumn = this.sortIndex+1;
-  @     }
-  @     for (i=0;i<newRows.length;i++) {
-  @       this.tbody[0].appendChild(newRows[i]);
-  @     }
-  @     this.setHdrIcons();
-  @   }
-  @   this.setHdrIcons = function() {
-  @     for (var i=0; i<this.hdrRow.cells.length; i++) {
-  @       if( this.columnTypes[i]=='x' ) continue;
-  @       var sortType;
-  @       if( this.prevColumn==i+1 ){
-  @         sortType = 'asc';
-  @       }else if( this.prevColumn==(-1-i) ){
-  @         sortType = 'desc'
-  @       }else{
-  @         sortType = 'none';
-  @       }
-  @       var hdrCell = this.hdrRow.cells[i];
-  @       var clsName = hdrCell.className.replace(/\s*\bsort\s*\w+/, '');
-  @       clsName += ' sort ' + sortType;
-  @       hdrCell.className = clsName;
-  @     }
-  @   }
-  @   this.sortText = function(a,b) {
-  @     var i = thisObject.sortIndex;
-  @     aa = a.cells[i].textContent.replace(/^\W+/,'').toLowerCase();
-  @     bb = b.cells[i].textContent.replace(/^\W+/,'').toLowerCase();
-  @     if(aa<bb) return -1;
-  @     if(aa==bb) return a.rowIndex-b.rowIndex;
-  @     return 1;
-  @   }
-  if( strchr(zColumnTypes,'T') ){
-    @   this.sortReverseText = function(a,b) {
-    @     var i = thisObject.sortIndex;
-    @     aa = a.cells[i].textContent.replace(/^\W+/,'').toLowerCase();
-    @     bb = b.cells[i].textContent.replace(/^\W+/,'').toLowerCase();
-    @     if(aa<bb) return +1;
-    @     if(aa==bb) return a.rowIndex-b.rowIndex;
-    @     return -1;
-    @   }
-  }
-  if( strchr(zColumnTypes,'n') ){
-    @   this.sortNumeric = function(a,b) {
-    @     var i = thisObject.sortIndex;
-    @     aa = parseFloat(a.cells[i].textContent);
-    @     if (isNaN(aa)) aa = 0;
-    @     bb = parseFloat(b.cells[i].textContent);
-    @     if (isNaN(bb)) bb = 0;
-    @     if(aa==bb) return a.rowIndex-b.rowIndex;
-    @     return aa-bb;
-    @   }
-  }
-  if( strchr(zColumnTypes,'N') ){
-    @   this.sortReverseNumeric = function(a,b) {
-    @     var i = thisObject.sortIndex;
-    @     aa = parseFloat(a.cells[i].textContent);
-    @     if (isNaN(aa)) aa = 0;
-    @     bb = parseFloat(b.cells[i].textContent);
-    @     if (isNaN(bb)) bb = 0;
-    @     if(aa==bb) return a.rowIndex-b.rowIndex;
-    @     return bb-aa;
-    @   }
-  }
-  if( strchr(zColumnTypes,'k') ){
-    @   this.sortKey = function(a,b) {
-    @     var i = thisObject.sortIndex;
-    @     aa = a.cells[i].getAttribute("data-sortkey");
-    @     bb = b.cells[i].getAttribute("data-sortkey");
-    @     if(aa<bb) return -1;
-    @     if(aa==bb) return a.rowIndex-b.rowIndex;
-    @     return 1;
-    @   }
-  }
-  if( strchr(zColumnTypes,'K') ){
-    @   this.sortReverseKey = function(a,b) {
-    @     var i = thisObject.sortIndex;
-    @     aa = a.cells[i].getAttribute("data-sortkey");
-    @     bb = b.cells[i].getAttribute("data-sortkey");
-    @     if(aa<bb) return +1;
-    @     if(aa==bb) return a.rowIndex-b.rowIndex;
-    @     return -1;
-    @   }
-  }
-  @   var x = tableEl.getElementsByTagName('thead');
-  @   if(!(this.tbody && this.tbody[0].rows && this.tbody[0].rows.length>0)){
-  @     return;
-  @   }
-  @   if(x && x[0].rows && x[0].rows.length > 0) {
-  @     this.hdrRow = x[0].rows[0];
-  @   } else {
-  @     return;
-  @   }
-  @   var thisObject = this;
-  @   this.prevColumn = initSort;
-  @   for (var i=0; i<this.hdrRow.cells.length; i++) {
-  @     if( columnTypes[i]=='x' ) continue;
-  @     var hdrcell = this.hdrRow.cells[i];
-  @     hdrcell.sTable = this;
-  @     hdrcell.style.cursor = "pointer";
-  @     hdrcell.sortType = columnTypes[i] || 't';
-  @     hdrcell.onclick = function () {
-  @       this.sTable.sort(this);
-  @       return false;
-  @     }
-  @   }
-  @   this.setHdrIcons()
-  @ }
-  @ var t = new SortableTable(gebi("%s(zTableId)"),"%s(zColumnTypes)",%d(iInitSort));
-  @ </script>
-}
-
-
 /*
 ** WEBPAGE: rptview
 **
@@ -1208,13 +1031,13 @@ void rptview_page(void){
       style_submenu_element("SQL", "rptsql?rn=%d",rn);
     }
     if( g.perm.NewTkt ){
-      style_submenu_element("New Ticket", "%s/tktnew", g.zTop);
+      style_submenu_element("New Ticket", "%R/tktnew");
     }
     style_header("%s", zTitle);
     output_color_key(zClrKey, 1,
         "border=\"0\" cellpadding=\"3\" cellspacing=\"0\" class=\"report\"");
-    @ <table border="1" cellpadding="2" cellspacing="0" class="report"
-    @  id="reportTable">
+    @ <table border="1" cellpadding="2" cellspacing="0" class="report sortable"
+    @  data-column-types='' data-init-sort='0'>
     sState.rn = rn;
     sState.nCount = 0;
     report_restrict_sql(&zErr1);
@@ -1226,7 +1049,7 @@ void rptview_page(void){
     }else if( zErr2 ){
       @ <p class="reportError">Error: %h(zErr2)</p>
     }
-    output_table_sorting_javascript("reportTable","",0);
+    style_table_sorter();
     style_footer();
   }else{
     report_restrict_sql(&zErr1);
