@@ -29,10 +29,12 @@ void setup_incr_cfgcnt(void){
   static int once = 1;
   if( once ){
     once = 0;
+    db_unprotect(PROTECT_CONFIG);
     db_multi_exec("UPDATE config SET value=value+1 WHERE name='cfgcnt'");
     if( db_changes()==0 ){
       db_multi_exec("INSERT INTO config(name,value) VALUES('cfgcnt',1)");
     }
+    db_protect_pop();
   }
 }
 
@@ -197,7 +199,10 @@ void onoff_attribute(
     int iQ = fossil_strcmp(zQ,"on")==0 || atoi(zQ);
     if( iQ!=iVal ){
       login_verify_csrf_secret();
+      db_protect_only(PROTECT_NONE);
       db_set(zVar, iQ ? "1" : "0", 0);
+      db_protect_pop();
+      setup_incr_cfgcnt();
       admin_log("Set option [%q] to [%q].",
                 zVar, iQ ? "on" : "off");
       iVal = iQ;
@@ -230,7 +235,10 @@ void entry_attribute(
   if( zQ && fossil_strcmp(zQ,zVal)!=0 ){
     const int nZQ = (int)strlen(zQ);
     login_verify_csrf_secret();
+    setup_incr_cfgcnt();
+    db_protect_only(PROTECT_NONE);
     db_set(zVar, zQ, 0);
+    db_protect_pop();
     admin_log("Set entry_attribute %Q to: %.*s%s",
               zVar, 20, zQ, (nZQ>20 ? "..." : ""));
     zVal = zQ;
@@ -260,7 +268,10 @@ const char *textarea_attribute(
   if( zQ && !disabled && fossil_strcmp(zQ,z)!=0){
     const int nZQ = (int)strlen(zQ);
     login_verify_csrf_secret();
+    db_protect_only(PROTECT_NONE);
     db_set(zVar, zQ, 0);
+    db_protect_pop();
+    setup_incr_cfgcnt();
     admin_log("Set textarea_attribute %Q to: %.*s%s",
               zVar, 20, zQ, (nZQ>20 ? "..." : ""));
     z = zQ;
@@ -296,7 +307,10 @@ void multiple_choice_attribute(
   if( zQ && fossil_strcmp(zQ,z)!=0){
     const int nZQ = (int)strlen(zQ);
     login_verify_csrf_secret();
+    db_unprotect(PROTECT_ALL);
     db_set(zVar, zQ, 0);
+    setup_incr_cfgcnt();
+    db_protect_pop();
     admin_log("Set multiple_choice_attribute %Q to: %.*s%s",
               zVar, 20, zQ, (nZQ>20 ? "..." : ""));
     z = zQ;
@@ -329,7 +343,7 @@ void setup_access(void){
 
   style_header("Access Control Settings");
   db_begin_transaction();
-  @ <form action="%s(g.zTop)/setup_access" method="post"><div>
+  @ <form action="%R/setup_access" method="post"><div>
   login_insert_csrf_secret();
   @ <input type="submit"  name="submit" value="Apply Changes" /></p>
   @ <hr />
@@ -606,7 +620,7 @@ void setup_login_group(void){
     @ is not currently part of any login-group.
     @ To join a login group, fill out the form below.</p>
     @
-    @ <form action="%s(g.zTop)/setup_login_group" method="post"><div>
+    @ <form action="%R/setup_login_group" method="post"><div>
     login_insert_csrf_secret();
     @ <blockquote><table border="0">
     @
@@ -662,7 +676,7 @@ void setup_login_group(void){
     db_finalize(&q);
     @ </table>
     @
-    @ <p><form action="%s(g.zTop)/setup_login_group" method="post"><div>
+    @ <p><form action="%R/setup_login_group" method="post"><div>
     login_insert_csrf_secret();
     @ To leave this login group press
     @ <input type="submit" value="Leave Login Group" name="leave">
@@ -719,7 +733,7 @@ void setup_timeline(void){
 
   style_header("Timeline Display Preferences");
   db_begin_transaction();
-  @ <form action="%s(g.zTop)/setup_timeline" method="post"><div>
+  @ <form action="%R/setup_timeline" method="post"><div>
   login_insert_csrf_secret();
   @ <p><input type="submit"  name="submit" value="Apply Changes" /></p>
 
@@ -865,7 +879,7 @@ void setup_settings(void){
   @ If the file for a versionable setting exists, the value cannot be
   @ changed on this screen.</p><hr /><p>
   @
-  @ <form action="%s(g.zTop)/setup_settings" method="post"><div>
+  @ <form action="%R/setup_settings" method="post"><div>
   @ <table border="0"><tr><td valign="top">
   login_insert_csrf_secret();
   for(i=0, pSet=aSetting; i<nSetting; i++, pSet++){
@@ -941,7 +955,7 @@ void setup_config(void){
 
   style_header("WWW Configuration");
   db_begin_transaction();
-  @ <form action="%s(g.zTop)/setup_config" method="post"><div>
+  @ <form action="%R/setup_config" method="post"><div>
   login_insert_csrf_secret();
   @ <input type="submit"  name="submit" value="Apply Changes" /></p>
   @ <hr />
@@ -1037,7 +1051,7 @@ void setup_wiki(void){
 
   style_header("Wiki Configuration");
   db_begin_transaction();
-  @ <form action="%s(g.zTop)/setup_wiki" method="post"><div>
+  @ <form action="%R/setup_wiki" method="post"><div>
   login_insert_csrf_secret();
   @ <input type="submit"  name="submit" value="Apply Changes" /></p>
   @ <hr />
@@ -1055,19 +1069,35 @@ void setup_wiki(void){
   @ </ul>
   @ (Property: "wiki-about")</p>
   @ <hr />
-  onoff_attribute("Enable WYSIWYG Wiki Editing",
-                  "wysiwyg-wiki", "wysiwyg-wiki", 0, 0);
-  @ <p>Enable what-you-see-is-what-you-get (WYSIWYG) editing of wiki pages.
-  @ The WYSIWYG editor generates HTML instead of markup, which makes
-  @ subsequent manual editing more difficult.
-  @ (Property: "wysiwyg-wiki")</p>
+  entry_attribute("Allow Unsafe HTML In Markdown", 6,
+                  "safe-html", "safe-html", "", 0);
+  @ <p>Allow "unsafe" HTML (ex: &lt;script&gt;, &lt;form&gt;, etc) to be
+  @ generated by <a href="%R/md_rules">Markdown-formatted</a> documents.
+  @ This setting is a string where each character indicates a "type" of
+  @ document in which to allow unsafe HTML:
+  @ <ul>
+  @ <li> <b>b</b> &rarr; checked-in files, embedded documentation
+  @ <li> <b>f</b> &rarr; forum posts
+  @ <li> <b>t</b> &rarr; tickets
+  @ <li> <b>w</b> &rarr; wiki pages
+  @ </ul>
+  @ Include letters for each type of document for which unsafe HTML should
+  @ be allowed.  For example, to allow unsafe HTML only for checked-in files,
+  @ make this setting be just "<b>b</b>".  To allow unsafe HTML anywhere except
+  @ in forum posts, make this setting be "<b>btw</b>".  The default is an
+  @ empty string which means that Fossil never allows Markdown documents
+  @ to generate unsafe HTML.
+  @ (Property: "safe-html")</p>
+  @ <hr />
+  @ The current interwiki tag map is as follows:
+  interwiki_append_map_table(cgi_output_blob());
+  @ <p>Visit <a href="./intermap">%R/intermap</a> for details or to
+  @ modify the interwiki tag map.
   @ <hr />
   onoff_attribute("Use HTML as wiki markup language",
     "wiki-use-html", "wiki-use-html", 0, 0);
   @ <p>Use HTML as the wiki markup language. Wiki links will still be parsed
-  @ but all other wiki formatting will be ignored. This option is helpful
-  @ if you have chosen to use a rich HTML editor for wiki markup such as
-  @ TinyMCE.</p>
+  @ but all other wiki formatting will be ignored.</p>
   @ <p><strong>CAUTION:</strong> when
   @ enabling, <i>all</i> HTML tags and attributes are accepted in the wiki.
   @ No sanitization is done. This means that it is very possible for malicious
@@ -1144,12 +1174,16 @@ void setup_adunit(void){
   }
   db_begin_transaction();
   if( P("clear")!=0 && cgi_csrf_safe(1) ){
+    db_unprotect(PROTECT_CONFIG);
     db_multi_exec("DELETE FROM config WHERE name GLOB 'adunit*'");
+    db_protect_pop();
     cgi_replace_parameter("adunit","");
+    cgi_replace_parameter("adright","");
+    setup_incr_cfgcnt();
   }
 
   style_header("Edit Ad Unit");
-  @ <form action="%s(g.zTop)/setup_adunit" method="post"><div>
+  @ <form action="%R/setup_adunit" method="post"><div>
   login_insert_csrf_secret();
   @ <b>Banner Ad-Unit:</b><br />
  textarea_attribute("", 6, 80, "adunit", "adunit", "", 0);
@@ -1210,7 +1244,7 @@ void setup_adunit(void){
 /*
 ** WEBPAGE: setup_logo
 **
-** Administrative page for changing the logo image.
+** Administrative page for changing the logo, background, and icon images.
 */
 void setup_logo(void){
   const char *zLogoMtime = db_get_mtime("logo-image", 0, 0);
@@ -1221,12 +1255,19 @@ void setup_logo(void){
   const char *zBgMime = db_get("background-mimetype","image/gif");
   const char *aBgImg = P("bgim");
   int szBgImg = atoi(PD("bgim:bytes","0"));
+  const char *zIconMtime = db_get_mtime("icon-image", 0, 0);
+  const char *zIconMime = db_get("icon-mimetype","image/gif");
+  const char *aIconImg = P("iconim");
+  int szIconImg = atoi(PD("iconim:bytes","0"));
   if( szLogoImg>0 ){
     zLogoMime = PD("logoim:mimetype","image/gif");
   }
   if( szBgImg>0 ){
     zBgMime = PD("bgim:mimetype","image/gif");
   }
+  if( szIconImg>0 ){
+    zIconMime = PD("iconim:mimetype","image/gif");
+  }
   login_check_credentials();
   if( !g.perm.Admin ){
     login_needed(0);
@@ -1239,6 +1280,7 @@ void setup_logo(void){
     Blob img;
     Stmt ins;
     blob_init(&img, aLogoImg, szLogoImg);
+    db_unprotect(PROTECT_CONFIG);
     db_prepare(&ins,
         "REPLACE INTO config(name,value,mtime)"
         " VALUES('logo-image',:bytes,now())"
@@ -1250,19 +1292,23 @@ void setup_logo(void){
        "REPLACE INTO config(name,value,mtime) VALUES('logo-mimetype',%Q,now())",
        zLogoMime
     );
+    db_protect_pop();
     db_end_transaction(0);
     cgi_redirect("setup_logo");
   }else if( P("clrlogo")!=0 ){
+    db_unprotect(PROTECT_CONFIG);
     db_multi_exec(
        "DELETE FROM config WHERE name IN "
            "('logo-image','logo-mimetype')"
     );
+    db_protect_pop();
     db_end_transaction(0);
     cgi_redirect("setup_logo");
   }else if( P("setbg")!=0 && zBgMime && zBgMime[0] && szBgImg>0 ){
     Blob img;
     Stmt ins;
     blob_init(&img, aBgImg, szBgImg);
+    db_unprotect(PROTECT_CONFIG);
     db_prepare(&ins,
         "REPLACE INTO config(name,value,mtime)"
         " VALUES('background-image',:bytes,now())"
@@ -1275,24 +1321,56 @@ void setup_logo(void){
        " VALUES('background-mimetype',%Q,now())",
        zBgMime
     );
+    db_protect_pop();
     db_end_transaction(0);
     cgi_redirect("setup_logo");
   }else if( P("clrbg")!=0 ){
+    db_unprotect(PROTECT_CONFIG);
     db_multi_exec(
        "DELETE FROM config WHERE name IN "
            "('background-image','background-mimetype')"
     );
+    db_protect_pop();
+    db_end_transaction(0);
+    cgi_redirect("setup_logo");
+  }else if( P("seticon")!=0 && zIconMime && zIconMime[0] && szIconImg>0 ){
+    Blob img;
+    Stmt ins;
+    blob_init(&img, aIconImg, szIconImg);
+    db_unprotect(PROTECT_CONFIG);
+    db_prepare(&ins,
+        "REPLACE INTO config(name,value,mtime)"
+        " VALUES('icon-image',:bytes,now())"
+    );
+    db_bind_blob(&ins, ":bytes", &img);
+    db_step(&ins);
+    db_finalize(&ins);
+    db_multi_exec(
+       "REPLACE INTO config(name,value,mtime)"
+       " VALUES('icon-mimetype',%Q,now())",
+       zIconMime
+    );
+    db_protect_pop();
+    db_end_transaction(0);
+    cgi_redirect("setup_logo");
+  }else if( P("clricon")!=0 ){
+    db_unprotect(PROTECT_CONFIG);
+    db_multi_exec(
+       "DELETE FROM config WHERE name IN "
+           "('icon-image','icon-mimetype')"
+    );
+    db_protect_pop();
     db_end_transaction(0);
     cgi_redirect("setup_logo");
   }
   style_header("Edit Project Logo And Background");
   @ <p>The current project logo has a MIME-Type of <b>%h(zLogoMime)</b>
   @ and looks like this:</p>
-  @ <blockquote><p><img src="%s(g.zTop)/logo/%z(zLogoMtime)" \
+  @ <blockquote><p><img src="%R/logo/%z(zLogoMtime)" \
   @ alt="logo" border="1" />
   @ </p></blockquote>
   @
-  @ <form action="%s(g.zTop)/setup_logo" method="post"
+  @ <form action="%R/setup_logo" method="post"
   @  enctype="multipart/form-data"><div>
   @ <p>The logo is accessible to all users at this URL:
   @ <a href="%s(g.zBaseURL)/logo">%s(g.zBaseURL)/logo</a>.
@@ -1312,11 +1390,11 @@ void setup_logo(void){
   @
   @ <p>The current background image has a MIME-Type of <b>%h(zBgMime)</b>
   @ and looks like this:</p>
-  @ <blockquote><p><img src="%s(g.zTop)/background/%z(zBgMtime)" \
+  @ <blockquote><p><img src="%R/background/%z(zBgMtime)" \
   @ alt="background" border=1 />
   @ </p></blockquote>
   @
-  @ <form action="%s(g.zTop)/setup_logo" method="post"
+  @ <form action="%R/setup_logo" method="post"
   @  enctype="multipart/form-data"><div>
   @ <p>The background image is accessible to all users at this URL:
   @ <a href="%s(g.zBaseURL)/background">%s(g.zBaseURL)/background</a>.
@@ -1334,6 +1412,30 @@ void setup_logo(void){
   @ <p>(Properties: "background-image" and "background-mimetype")
   @ <hr />
   @
+  @ <p>The current icon image has a MIME-Type of <b>%h(zIconMime)</b>
+  @ and looks like this:</p>
+  @ <blockquote><p><img src="%R/favicon.ico/%z(zIconMtime)" \
+  @ alt="icon" border=1 />
+  @ </p></blockquote>
+  @
+  @ <form action="%R/setup_logo" method="post"
+  @  enctype="multipart/form-data"><div>
+  @ <p>The icon image is accessible to all users at this URL:
+  @ <a href="%s(g.zBaseURL)/favicon.ico">%s(g.zBaseURL)/favicon.ico</a>.
+  @ The icon image may or may not appear on each
+  @ page depending on the web browser in use and the MIME-Types that it
+  @ supports for icon images.
+  @ To change the icon image, use the following form:</p>
+  login_insert_csrf_secret();
+  @ Icon image file:
+  @ <input type="file" name="iconim" size="60" accept="image/*" />
+  @ <p align="center">
+  @ <input type="submit" name="seticon" value="Change Icon" />
+  @ <input type="submit" name="clricon" value="Revert To Default" /></p>
+  @ </div></form>
+  @ <p>(Properties: "icon-image" and "icon-mimetype")
+  @ <hr />
+  @
   @ <p><span class="note">Note:</span>  Your browser has probably cached these
   @ images, so you may need to press the Reload button before changes will
   @ take effect. </p>
@@ -1414,7 +1516,7 @@ void sql_page(void){
      go = 1;
   }
   @
-  @ <form method="post" action="%s(g.zTop)/admin_sql">
+  @ <form method="post" action="%R/admin_sql">
   login_insert_csrf_secret();
   @ SQL:<br />
   @ <textarea name="q" rows="8" cols="80">%h(zQ)</textarea><br />
@@ -1425,12 +1527,12 @@ void sql_page(void){
   @ </form>
   if( P("schema") ){
     zQ = sqlite3_mprintf(
-            "SELECT sql FROM repository.sqlite_master"
+            "SELECT sql FROM repository.sqlite_schema"
             " WHERE sql IS NOT NULL ORDER BY name");
     go = 1;
   }else if( P("tablelist") ){
     zQ = sqlite3_mprintf(
-            "SELECT name FROM repository.sqlite_master WHERE type='table'"
+            "SELECT name FROM repository.sqlite_schema WHERE type='table'"
             " ORDER BY name");
     go = 1;
   }
@@ -1523,7 +1625,7 @@ void th1_page(void){
   @ run by this page.  If Tcl integration was enabled at compile-time and
   @ the "tcl" setting is enabled, Tcl commands may be run as well.</p>
   @
-  @ <form method="post" action="%s(g.zTop)/admin_th1">
+  @ <form method="post" action="%R/admin_th1">
   login_insert_csrf_secret();
   @ TH1:<br />
   @ <textarea name="q" rows="5" cols="80">%h(zQ)</textarea><br />
@@ -1625,7 +1727,7 @@ void page_srchsetup(){
     return;
   }
   style_header("Search Configuration");
-  @ <form action="%s(g.zTop)/srchsetup" method="post"><div>
+  @ <form action="%R/srchsetup" method="post"><div>
   login_insert_csrf_secret();
   @ <div style="text-align:center;font-weight:bold;">
   @ Server-specific settings that affect the
@@ -1774,7 +1876,9 @@ void page_waliassetup(){
     sqlite3_snprintf(sizeof(zCnt), zCnt, "v%d", cnt);
     zValue = PD(zCnt,"");
     setup_update_url_alias(&sql, "", zNewName, zValue);
+    db_unprotect(PROTECT_CONFIG);
     db_multi_exec("%s", blob_sql_text(&sql));
+    db_protect_pop();
     blob_reset(&sql);
     blob_reset(&namelist);
     cnt = 0;
@@ -1783,7 +1887,7 @@ void page_waliassetup(){
       "SELECT substr(name,8), value FROM config WHERE name GLOB 'walias:/*'"
       " UNION ALL SELECT '', ''"
   );
-  @ <form action="%s(g.zTop)/waliassetup" method="post"><div>
+  @ <form action="%R/waliassetup" method="post"><div>
   login_insert_csrf_secret();
   @ <table border=0 cellpadding=5>
   @ <tr><th>Alias<th>URI That The Alias Maps Into
