@@ -317,7 +317,14 @@ void user_edit(void){
     int n;
     if( P("verifydelete") ){
       /* Verified delete user request */
+      db_unprotect(PROTECT_USER);
+      if( db_table_exists("repository","subscriber") ){
+        /* Also delete any subscriptions associated with this user */
+        db_multi_exec("DELETE FROM subscriber WHERE suname="
+                      "(SELECT login FROM user WHERE uid=%d)", uid);
+      }
       db_multi_exec("DELETE FROM user WHERE uid=%d", uid);
+      db_protect_pop();
       moderation_disapprove_for_missing_users();
       admin_log("Deleted user [%s] (uid %d).",
                 PD("login","???")/*safe-for-%s*/, uid);
@@ -403,11 +410,13 @@ void user_edit(void){
       return;
     }
     login_verify_csrf_secret();
+    db_unprotect(PROTECT_USER);
     db_multi_exec(
        "REPLACE INTO user(uid,login,info,pw,cap,mtime) "
        "VALUES(nullif(%d,0),%Q,%Q,%Q,%Q,now())",
       uid, zLogin, P("info"), zPw, zCap
     );
+    db_protect_pop();
     setup_incr_cfgcnt();
     admin_log( "Updated user [%q] with capabilities [%q].",
                zLogin, zCap );
@@ -434,7 +443,9 @@ void user_edit(void){
         zLogin, P("pw"), zLogin, P("info"), zCap,
         zOldLogin
       );
+      db_unprotect(PROTECT_USER);
       login_group_sql(blob_str(&sql), "<li> ", " </li>\n", &zErr);
+      db_protect_pop();
       blob_reset(&sql);
       admin_log( "Updated user [%q] in all login groups "
                  "with capabilities [%q].",
@@ -704,7 +715,7 @@ void user_edit(void){
   @ </table>
   @ </div></form>
   @ </div>
-  style_load_one_js_file("useredit.js");
+  builtin_request_js("useredit.js");
   @ <hr>
   @ <h1>Notes On Privileges And Capabilities:</h1>
   @ <ul>
