@@ -31,36 +31,75 @@
 */
 void sitemap_page(void){
   int srchFlags;
+  int inSublist = 0;
+  int i;
+  int isPopup = 0;         /* This is an XMLHttpRequest() for /sitemap */
+  const struct {
+    const char *zTitle;
+    const char *zProperty;
+  } aExtra[] = {
+    { "Documentation",  "sitemap-docidx" },
+    { "Download",       "sitemap-download" },
+    { "License",        "sitemap-license" },
+    { "Contact",        "sitemap-contact" },
+  };
+
   login_check_credentials();
+  if( P("popup")!=0 && cgi_csrf_safe(0) ){
+    /* If this is a POST from the same origin with the popup=1 parameter,
+    ** then disable anti-robot defenses */
+    isPopup = 1;
+    g.perm.Hyperlink = 1;
+    g.javascriptHyperlink = 0;
+  }
   srchFlags = search_restrict(SRCH_ALL);
-  style_header("Site Map");
-  style_adunit_config(ADUNIT_RIGHT_OK);
-#if 0
-  @ <p>
-  @ The following links are just a few of the many web-pages available for
-  @ this Fossil repository:
-  @ </p>
-  @
-#endif
-  @ <ul>
+  if( !isPopup ){
+    style_header("Site Map");
+    style_adunit_config(ADUNIT_RIGHT_OK);
+  }
+  @ <ul id="sitemap" class="columns" style="column-width:20em">
   @ <li>%z(href("%R/home"))Home Page</a>
+  for(i=0; i<sizeof(aExtra)/sizeof(aExtra[0]); i++){
+    char *z = db_get(aExtra[i].zProperty,0);
+    if( z==0 || z[0]==0 ) continue;
+    if( !inSublist ){
+      @ <ul>
+      inSublist = 1;
+    }
+    if( z[0]=='/' ){
+      @ <li>%z(href("%R%s",z))%s(aExtra[i].zTitle)</a></li>
+    }else{
+      @ <li>%z(href("%s",z))%s(aExtra[i].zTitle)</a></li>
+    }
+  }
   if( srchFlags & SRCH_DOC ){
-    @   <ul>
-    @   <li>%z(href("%R/docsrch"))Search Project Documentation</a></li>
-    @   </ul>
+    if( !inSublist ){
+      @ <ul>
+      inSublist = 1;
+    }
+    @ <li>%z(href("%R/docsrch"))Documentation Search</a></li>
+  }
+  if( inSublist ){
+    @ </ul>
+    inSublist = 0;    
   }
   @ </li>
   if( g.perm.Read ){
-    @ <li>%z(href("%R/tree"))File Browser</a></li>
+    const char *zEditGlob = db_get("fileedit-glob","");
+    @ <li>%z(href("%R/tree"))File Browser</a>
     @   <ul>
     @   <li>%z(href("%R/tree?type=tree&ci=trunk"))Tree-view,
     @        Trunk Check-in</a></li>
     @   <li>%z(href("%R/tree?type=flat"))Flat-view</a></li>
     @   <li>%z(href("%R/fileage?name=trunk"))File ages for Trunk</a></li>
+    @   <li>%z(href("%R/uvlist"))Unversioned Files</a>
+    if( g.perm.Write && zEditGlob[0]!=0 ){
+      @   <li>%z(href("%R/fileedit"))On-line File Editor</li>
+    }
     @ </ul>
   }
   if( g.perm.Read ){
-    @ <li>%z(href("%R/timeline?n=200"))Project Timeline</a></li>
+    @ <li>%z(href("%R/timeline"))Project Timeline</a>
     @ <ul>
     @   <li>%z(href("%R/reports"))Activity Reports</a></li>
     @   <li>%z(href("%R/timeline?n=all&namechng"))File name changes</a></li>
@@ -68,15 +107,37 @@ void sitemap_page(void){
     @   <li>%z(href("%R/timeline?a=1970-01-01&y=ci&n=10"))First 10
     @       check-ins</a></li>
     @ </ul>
+    @ </li>
   }
   if( g.perm.Read ){
-    @ <li>%z(href("%R/brlist"))Branches</a></li>
+    @ <li>%z(href("%R/brlist"))Branches</a>
     @ <ul>
+    @   <li>%z(href("%R/taglist"))Tags</a></li>
     @   <li>%z(href("%R/leaves"))Leaf Check-ins</a></li>
-    @   <li>%z(href("%R/taglist"))List of Tags</a></li>
     @ </ul>
     @ </li>
   }
+  if( srchFlags ){
+    @ <li>%z(href("%R/search"))Search</a></li>
+  }
+  if( g.perm.RdForum ){
+    @ <li>%z(href("%R/forum"))Forum</a>
+    @ <ul>
+    @   <li>%z(href("%R/timeline?y=f"))Recent activity</a></li>
+    @ </ul>
+    @ </li>
+  }
+  if( g.perm.RdTkt ){
+    @ <li>%z(href("%R/reportlist"))Tickets/Bug Reports</a>
+    @   <ul>
+    if( srchFlags & SRCH_TKT ){
+      @   <li>%z(href("%R/tktsrch"))Ticket Search</a></li>
+    }
+    @   <li>%z(href("%R/timeline?y=t"))Recent activity</a></li>
+    @   <li>%z(href("%R/attachlist"))List of Attachments</a></li>
+    @   </ul>
+    @ </li>
+  }
   if( g.perm.RdWiki ){
     @ <li>%z(href("%R/wikihelp"))Wiki</a>
     @   <ul>
@@ -85,33 +146,49 @@ void sitemap_page(void){
     }
     @     <li>%z(href("%R/wcontent"))List of Wiki Pages</a></li>
     @     <li>%z(href("%R/timeline?y=w"))Recent activity</a></li>
-    @     <li>%z(href("%R/wiki_rules"))Wiki Formatting Rules</a></li>
-    @     <li>%z(href("%R/md_rules"))Markdown Formatting Rules</a></li>
-    @     <li>%z(href("%R/wiki?name=Sandbox"))Sandbox</a></li>
+    @     <li>%z(href("%R/wikiedit?name=Sandbox"))Wiki Sandbox</a></li>
     @     <li>%z(href("%R/attachlist"))List of Attachments</a></li>
+    @     <li>%z(href("%R/pikchrshow"))Pikchr Sandbox</a></li>
     @   </ul>
     @ </li>
   }
-  if( g.perm.RdTkt ){
-    @ <li>%z(href("%R/reportlist"))Tickets</a>
-    @   <ul>
-    if( srchFlags & SRCH_TKT ){
-      @   <li>%z(href("%R/tktsrch"))Ticket Search</a></li>
+
+  if( !g.zLogin ){
+    @ <li>%z(href("%R/login"))Login</a>
+    if( login_self_register_available(0) ){
+       @ <ul>
+       @ <li>%z(href("%R/register"))Create a new account</a></li>
+       inSublist = 1;
+    }
+  }else {
+    @ <li>%z(href("%R/logout"))Logout</a>
+    if( g.perm.Password ){
+      @ <ul>
+      @ <li>%z(href("%R/logout"))Change Password</a></li>
+      inSublist = 1;
     }
-    @   <li>%z(href("%R/timeline?y=t"))Recent activity</a></li>
-    @   <li>%z(href("%R/attachlist"))List of Attachments</a></li>
-    @   </ul>
-    @ </li>
   }
-  if( srchFlags ){
-    @ <li>%z(href("%R/search"))Full-Text Search</a></li>
+  if( alert_enabled() && g.perm.EmailAlert ){
+    if( !inSublist ){
+      inSublist = 1;
+      @ <ul>
+    }
+    if( login_is_individual() ){
+      @ <li>%z(href("%R/alerts"))Email Alerts</a></li>
+    }else{
+      @ <li>%z(href("%R/subscribe"))Subscribe to Email Alerts</a></li>
+    }
   }
-  @ <li>%z(href("%R/login"))Login/Logout/Change Password</a></li>
+  if( inSublist ){
+    @ </ul>
+    inSublist = 0;
+  }
+  @ </li>
+
   if( g.perm.Read ){
     @ <li>%z(href("%R/stat"))Repository Status</a>
     @   <ul>
-    @   <li>%z(href("%R/hash-collisions"))Collisions on SHA1 hash
-    @       prefixes</a></li>
+    @   <li>%z(href("%R/hash-collisions"))Collisions on hash prefixes</a></li>
     if( g.perm.Admin ){
       @   <li>%z(href("%R/urllist"))List of URLs used to access
       @       this repository</a></li>
@@ -121,11 +198,20 @@ void sitemap_page(void){
     @   </ul>
     @ </li>
   }
-  @ <li>On-line Documentation
+  @ <li>%z(href("%R/help"))Help</a>
   @   <ul>
-  @   <li>%z(href("%R/help"))List of All Commands and Web Pages</a></li>
+  if( g.perm.Admin || g.perm.Write ||
+      g.perm.WrForum || g.perm.WrTForum ||
+      g.perm.NewWiki || g.perm.ApndWiki || g.perm.WrWiki || g.perm.ModWiki ||
+      g.perm.NewTkt  || g.perm.ApndTkt  || g.perm.WrTkt  || g.perm.ModTkt ){
+    @   <li>%z(href("%R/wiki_rules"))Wiki Formatting Rules</a></li>
+    @   <li>%z(href("%R/md_rules"))Markdown Formatting Rules</a></li>
+  }
   @   <li>%z(href("%R/test-all-help"))All "help" text on a single page</a></li>
-  @   <li>%z(href("%R/mimetype_list"))Filename suffix to mimetype map</a></li>
+  if( g.perm.Admin || g.perm.Write || g.perm.WrUnver ){
+    @   <li>%z(href("%R/mimetype_list"))\
+    @ Filename suffix to MIME type map</a></li>
+  }
   @   </ul></li>
   if( g.perm.Admin ){
     @ <li>%z(href("%R/setup"))Administration Pages</a>
@@ -147,6 +233,8 @@ void sitemap_page(void){
   @       colors assigned to branch names</a>
   @   <li>%z(href("%R/test-captcha"))Random ASCII-art Captcha image</a></li>
   @   </ul></li>
-  @ </ul></li>
-  style_footer();
+  @ </ul>
+  if( !isPopup ){
+    style_footer();
+  }
 }
