@@ -90,10 +90,6 @@ static int submenuEnable = 1;
 ** Flags for various javascript files needed prior to </body>
 */
 static int needHrefJs = 0;      /* href.js */
-static int needSortJs = 0;      /* sorttable.js */
-static int needGraphJs = 0;     /* graph.js */
-static int needCopyBtnJs = 0;   /* copybtn.js */
-static int needAccordionJs = 0; /* accordion.js */
 
 /*
 ** Extra JS added to the end of the file.
@@ -146,6 +142,7 @@ static Blob blobOnLoad = BLOB_INITIALIZER;
 char *xhref(const char *zExtra, const char *zFormat, ...){
   char *zUrl;
   va_list ap;
+  if( !g.perm.Hyperlink ) return fossil_strdup("");
   va_start(ap, zFormat);
   zUrl = vmprintf(zFormat, ap);
   va_end(ap);
@@ -170,6 +167,7 @@ char *xhref(const char *zExtra, const char *zFormat, ...){
 char *chref(const char *zExtra, const char *zFormat, ...){
   char *zUrl;
   va_list ap;
+  if( !g.perm.Hyperlink ) return fossil_strdup("");
   va_start(ap, zFormat);
   zUrl = vmprintf(zFormat, ap);
   va_end(ap);
@@ -185,6 +183,7 @@ char *chref(const char *zExtra, const char *zFormat, ...){
 char *href(const char *zFormat, ...){
   char *zUrl;
   va_list ap;
+  if( !g.perm.Hyperlink ) return fossil_strdup("");
   va_start(ap, zFormat);
   zUrl = vmprintf(zFormat, ap);
   va_end(ap);
@@ -342,8 +341,8 @@ static int submenuCompare(const void *a, const void *b){
   return fossil_strcmp(A->zLabel, B->zLabel);
 }
 
-/* Use this for the $current_page variable if it is not NULL.  If it is
-** NULL then use g.zPath.
+/* Use this for the $current_page variable if it is not NULL.  If it
+** is NULL then use g.zPath.
 */
 static char *local_zCurrentPage = 0;
 
@@ -363,8 +362,9 @@ void style_set_current_page(const char *zFormat, ...){
 }
 
 /*
-** Create a TH1 variable containing the URL for the specified config resource.
-** The resulting variable name will be of the form $[zVarPrefix]_url.
+** Create a TH1 variable containing the URL for the specified config
+** resource. The resulting variable name will be of the form
+** $[zVarPrefix]_url.
 */
 static void url_var(
   const char *zVarPrefix,
@@ -372,11 +372,25 @@ static void url_var(
   const char *zPageName
 ){
   char *zVarName = mprintf("%s_url", zVarPrefix);
-  char *zUrl = mprintf("%R/%s?id=%x", zPageName,
-                       skin_id(zConfigName));
+  char *zUrl = 0;              /* stylesheet URL */
+  int hasBuiltin = 0;          /* true for built-in page-specific CSS */
+
+  if(0==strcmp("css",zConfigName)){
+    /* Account for page-specific CSS, appending a /{{g.zPath}} to the
+    ** url only if we have a corresponding built-in page-specific CSS
+    ** file. Do not append it to all pages because we would
+    ** effectively cache-bust all pages which do not have
+    ** page-specific CSS. */
+    char * zBuiltin = mprintf("style.%s.css", g.zPath);
+    hasBuiltin = builtin_file(zBuiltin,0)!=0;
+    fossil_free(zBuiltin);
+  }
+  zUrl = mprintf("%R/%s%s%s?id=%x", zPageName,
+                 hasBuiltin ? "/" : "", hasBuiltin ? g.zPath : "",
+                 skin_id(zConfigName));
   Th_Store(zVarName, zUrl);
-  free(zUrl);
-  free(zVarName);
+  fossil_free(zUrl);
+  fossil_free(zVarName);
 }
 
 /*
@@ -471,7 +485,7 @@ char *style_copy_button(
     }
   }
   free(zText);
-  style_copybutton_control();
+  builtin_request_js("copybtn.js");
   return zResult;
 }
 
@@ -517,6 +531,7 @@ char *style_csp(int toHeader){
   Blob csp;
   char *zNonce;
   char *zCsp;
+  int i;
   if( zFormat[0]==0 ){
     zFormat = zBackupCSP;
   }
@@ -528,6 +543,9 @@ char *style_csp(int toHeader){
   }
   blob_append(&csp, zFormat, -1);
   zCsp = blob_str(&csp);
+  /* No whitespace other than actual space characters allowed in the CSP
+  ** string.  See https://fossil-scm.org/forum/forumpost/d29e3af43c */
+  for(i=0; zCsp[i]; i++){ if( fossil_isspace(zCsp[i]) ) zCsp[i] = ' '; }
   if( toHeader ){
     cgi_printf_header("Content-Security-Policy: %s\r\n", zCsp);
   }
@@ -539,7 +557,7 @@ char *style_csp(int toHeader){
 ** header template lacks a <body> tag, then all of the following is
 ** prepended.
 */
-static char zDfltHeader[] = 
+static const char zDfltHeader[] = 
 @ <html>
 @ <head>
 @ <base href="$baseurl/$current_page" />
@@ -553,6 +571,13 @@ static char zDfltHeader[] =
 @ <body>
 ;
 
+/*
+** Returns the default page header.
+*/
+const char *get_default_header(){
+  return zDfltHeader;
+}
+
 /*
 ** Initialize all the default TH1 variables
 */
@@ -681,107 +706,35 @@ static const char *style_adunit_text(unsigned int *pAdFlag){
 ** Indicate that the table-sorting javascript is needed.
 */
 void style_table_sorter(void){
-  needSortJs = 1;
-}
-
-/*
-** Indicate that the accordion javascript is needed.
-*/
-void style_accordion(void){
-  needAccordionJs = 1;
-}
-
-/*
-** Indicate that the timeline graph javascript is needed.
-*/
-void style_graph_generator(void){
-  needGraphJs = 1;
-}
-
-/*
-** Indicate that the copy button javascript is needed.
-*/
-void style_copybutton_control(void){
-  needCopyBtnJs = 1;
-}
-
-/*
-** Generate code to load a single javascript file
-*/
-void style_load_one_js_file(const char *zFile){
-  @ <script src='%R/builtin/%s(zFile)?id=%S(fossil_exe_id())'></script>
-}
-
-/*
-** All extra JS files to load.
-*/
-static const char *azJsToLoad[4];
-static int nJsToLoad = 0;
-
-/*
-** Register a new JS file to load at the end of the document.
-*/
-void style_load_js(const char *zName){
-  int i;
-  for(i=0; i<nJsToLoad; i++){
-    if( fossil_strcmp(zName, azJsToLoad[i])==0 ) return;
-  }
-  if( nJsToLoad>=sizeof(azJsToLoad)/sizeof(azJsToLoad[0]) ){
-    fossil_panic("too many JS files");
-  }
-  azJsToLoad[nJsToLoad++] = zName;
+  builtin_request_js("sorttable.js");
 }
 
 /*
 ** Generate code to load all required javascript files.
 */
 static void style_load_all_js_files(void){
-  int i;
-  if( needHrefJs ){
+  if( needHrefJs && g.perm.Hyperlink ){
     int nDelay = db_get_int("auto-hyperlink-delay",0);
     int bMouseover = db_get_boolean("auto-hyperlink-mouseover",0);
     @ <script id='href-data' type='application/json'>\
     @ {"delay":%d(nDelay),"mouseover":%d(bMouseover)}</script>
   }
-  @ <script nonce="%h(style_nonce())">
+  @ <script nonce="%h(style_nonce())">/* style.c:%d(__LINE__) */
   @ function debugMsg(msg){
   @ var n = document.getElementById("debugMsg");
   @ if(n){n.textContent=msg;}
   @ }
-  if( needHrefJs ){
+  if( needHrefJs && g.perm.Hyperlink ){
+    @ /* href.js */
     cgi_append_content(builtin_text("href.js"),-1);
   }
-  if( needSortJs ){
-    cgi_append_content(builtin_text("sorttable.js"),-1);
-  }
-  if( needGraphJs ){
-    cgi_append_content(builtin_text("graph.js"),-1);
-  }
-  if( needCopyBtnJs ){
-    cgi_append_content(builtin_text("copybtn.js"),-1);
-  }
-  if( needAccordionJs ){
-    cgi_append_content(builtin_text("accordion.js"),-1);
-  }
-  for(i=0; i<nJsToLoad; i++){
-    cgi_append_content(builtin_text(azJsToLoad[i]),-1);
-  }
   if( blob_size(&blobOnLoad)>0 ){
     @ window.onload = function(){
     cgi_append_content(blob_buffer(&blobOnLoad), blob_size(&blobOnLoad));
     cgi_append_content("\n}\n", -1);
   }
   @ </script>
-}
-
-/*
-** Extra JS to run after all content is loaded.
-*/
-void style_js_onload(const char *zFormat, ...){
-  va_list ap;
-  va_start(ap, zFormat);
-  blob_vappendf(&blobOnLoad, zFormat, ap);
-  va_end(ap);
+  builtin_fulfill_js_requests();
 }
 
 /*
@@ -903,7 +856,7 @@ void style_footer(void){
       cgi_query_parameters_to_hidden();
       cgi_tag_query_parameter(0);
       @ </form>
-      style_load_one_js_file("menu.js");
+      builtin_request_js("menu.js");
     }
   }
 
@@ -974,29 +927,6 @@ void style_sidebox_end(void){
   @ </div>
 }
 
-/*
-** Insert the cssDefaultList[] table, generated from default_css.txt
-** using the mkcss.c program.
-*/
-#include "default_css.h"
-
-/*
-** Append all of the default CSS to the CGI output.
-*/
-void cgi_append_default_css(void) {
-  int i;
-
-  cgi_printf("%s", builtin_text("skins/default/css.txt"));
-  for( i=0; cssDefaultList[i].elementClass; i++ ){
-    if( cssDefaultList[i].elementClass[0] ){
-      cgi_printf("%s {\n%s\n}\n\n",
-                 cssDefaultList[i].elementClass,
-                 cssDefaultList[i].value
-                );
-    }
-  }
-}
-
 /*
 ** Search string zCss for zSelector.
 **
@@ -1026,6 +956,11 @@ static int containsSelector(const char *zCss, const char *zSelector){
 ** Usage: %fossil test-contains-selector FILENAME SELECTOR
 **
 ** Determine if the CSS stylesheet FILENAME contains SELECTOR.
+**
+** Note that as of 2020-05-28, the default rules are always emitted,
+** so the containsSelector() logic is no longer applied when emitting
+** style.css. It is unclear whether this test command is now obsolete
+** or whether it may still serve a purpose.
 */
 void contains_selector_cmd(void){
   int found;
@@ -1058,6 +993,43 @@ void page_script_js(void){
   Th_Render(zScript?zScript:"");
 }
 
+/*
+** If one of the "name" or "page" URL parameters (in that order)
+** is set then this function looks for page/page group-specific
+** CSS and (if found) appends it to pOut, else it is a no-op.
+*/
+static void page_style_css_append_page_style(Blob *pOut){
+  const char *zPage = PD("name",P("page"));
+  char * zFile;
+  int nFile = 0;
+  const char *zBuiltin;
+
+  if(zPage==0 || zPage[0]==0){
+    return;
+  }
+  zFile = mprintf("style.%s.css", zPage);
+  zBuiltin = (const char *)builtin_file(zFile, &nFile);
+  if(nFile>0){
+    blob_appendf(pOut,
+      "\n/***********************************************************\n"
+      "** Start of page-specific CSS for page %s...\n"
+      "***********************************************************/\n",
+      zPage);
+    blob_append(pOut, zBuiltin, nFile);
+    blob_appendf(pOut,
+      "\n/***********************************************************\n"
+      "** End of page-specific CSS for page %s.\n"
+      "***********************************************************/\n",
+      zPage);
+    fossil_free(zFile);
+    return;
+  }
+  /* Potential TODO: check for aliases/page groups. e.g. group all
+  ** /forumXYZ CSS into one file, all /setupXYZ into another, etc. As
+  ** of this writing, doing so would only shave a few kb from
+  ** default.css. */
+  fossil_free(zFile);
+}
 
 /*
 ** WEBPAGE: style.css
@@ -1065,34 +1037,23 @@ void page_script_js(void){
 ** Return the style sheet.
 */
 void page_style_css(void){
-  Blob css;
+  Blob css = empty_blob;
   int i;
-  int isInit = 0;
+  const char * zDefaults;
 
   cgi_set_content_type("text/css");
-  blob_init(&css,skin_get("css"),-1);
-
-  /* add special missing definitions */
-  for(i=1; cssDefaultList[i].elementClass; i++){
-    char *z = blob_str(&css);
-    if( !containsSelector(z, cssDefaultList[i].elementClass) ){
-      if( !isInit ){
-        isInit = 1;
-        blob_append(&css,
-          "\n/***********************************************************\n"
-          "** All CSS above is supplied by the repository \"skin\".\n"
-          "** That which follows is generated automatically by Fossil\n"
-          "** to fill in needed selectors that are missing from the\n"
-          "** \"skin\" CSS.\n"
-          "***********************************************************/\n",
-          -1);
-      }
-      blob_appendf(&css, "%s {\n%s}\n",
-          cssDefaultList[i].elementClass,
-          cssDefaultList[i].value);
-    }
-  }
-
+  etag_check(0, 0);
+  /* Emit all default rules... */
+  zDefaults = (const char*)builtin_file("default.css", &i);
+  blob_append(&css, zDefaults, i);
+  /* Page-specific CSS, if any... */
+  page_style_css_append_page_style(&css);
+  blob_append(&css,
+     "\n/***********************************************************\n"
+     "** All CSS which follows is supplied by the repository \"skin\".\n"
+     "***********************************************************/\n",
+     -1);
+  blob_append(&css,skin_get("css"),-1);
   /* Process through TH1 in order to give an opportunity to substitute
   ** variables such as $baseurl.
   */
@@ -1107,42 +1068,6 @@ void page_style_css(void){
   g.isConst = 1;
 }
 
-/*
-** WEBPAGE: builtin
-** URL:  builtin/FILENAME
-**
-** Return the built-in text given by FILENAME.  This is used internally 
-** by many Fossil web pages to load built-in javascript files.
-**
-** If the id= query parameter is present, then Fossil assumes that the
-** result is immutable and sets a very large cache retention time (1 year).
-*/
-void page_builtin_text(void){
-  Blob out;
-  const char *zName = P("name");
-  const char *zTxt = 0;
-  const char *zId = P("id");
-  int nId;
-  if( zName ) zTxt = builtin_text(zName);
-  if( zTxt==0 ){
-    cgi_set_status(404, "Not Found");
-    @ File "%h(zName)" not found
-    return;
-  }
-  if( sqlite3_strglob("*.js", zName)==0 ){
-    cgi_set_content_type("application/javascript");
-  }else{
-    cgi_set_content_type("text/plain");
-  }
-  if( zId && (nId = (int)strlen(zId))>=8 && strncmp(zId,MANIFEST_UUID,nId)==0 ){
-    g.isConst = 1;
-  }else{
-    etag_check(0,0);
-  }
-  blob_init(&out, zTxt, -1);
-  cgi_set_content(&out);
-}
-
 /*
 ** All possible capabilities
 */
@@ -1298,3 +1223,245 @@ void webpage_assert_page(const char *zFile, int iLine, const char *zExpr){
 #if INTERFACE
 # define webpage_assert(T) if(!(T)){webpage_assert_page(__FILE__,__LINE__,#T);}
 #endif
+
+/*
+** Returns a pseudo-random input field ID, for use in associating an
+** ID-less input field with a label. The memory is owned by the
+** caller.
+*/
+static char * style_next_input_id(){
+  static int inputID = 0;
+  ++inputID;
+  return mprintf("input-id-%d", inputID);
+}
+
+/*
+** Outputs a labeled checkbox element. zWrapperId is an optional ID
+** value for the containing element (see below). zFieldName is the
+** form element name. zLabel is the label for the checkbox. zValue is
+** the optional value for the checkbox. zTip is an optional tooltip,
+** which gets set as the "title" attribute of the outermost
+** element. If isChecked is true, the checkbox gets the "checked"
+** attribute set, else it is not.
+**
+** Resulting structure:
+**
+** <div class='input-with-label' title={{zTip}} id={{zWrapperId}}>
+**   <input type='checkbox' name={{zFieldName}} value={{zValue}}
+**          id='A RANDOM VALUE'
+**          {{isChecked ? " checked : ""}}/>
+**   <label for='ID OF THE INPUT FIELD'>{{zLabel}}</label>
+** </div>
+**
+** zLabel, and zValue are required. zFieldName, zWrapperId, and zTip
+** are may be NULL or empty.
+**
+** Be sure that the input-with-label CSS class is defined sensibly, in
+** particular, having its display:inline-block is useful for alignment
+** purposes.
+*/
+void style_labeled_checkbox(const char * zWrapperId,
+                            const char *zFieldName, const char * zLabel,
+                            const char * zValue, int isChecked,
+                            const char * zTip){
+  char * zLabelID = style_next_input_id();
+  CX("<div class='input-with-label'");
+  if(zTip && *zTip){
+    CX(" title='%h'", zTip);
+  }
+  if(zWrapperId && *zWrapperId){
+    CX(" id='%s'",zWrapperId);
+  }
+  CX("><input type='checkbox' id='%s' ", zLabelID);
+  if(zFieldName && *zFieldName){
+    CX("name='%s' ",zFieldName);
+  }
+  CX("value='%T'%s/>",
+     zValue ? zValue : "", isChecked ? " checked" : "");
+  CX("<label for='%s'>%h</label></div>", zLabelID, zLabel);
+  fossil_free(zLabelID);
+}
+
+/*
+** Outputs a SELECT list from a compile-time list of integers.
+** The vargs must be a list of (const char *, int) pairs, terminated
+** with a single NULL. Each pair is interpreted as...
+**
+** If the (const char *) is NULL, it is the end of the list, else
+** a new OPTION entry is created. If the string is empty, the
+** label and value of the OPTION is the integer part of the pair.
+** If the string is not empty, it becomes the label and the integer
+** the value. If that value == selectedValue then that OPTION
+** element gets the 'selected' attribute.
+**
+** Note that the pairs are not in (int, const char *) order because
+** there is no well-known integer value which we can definitively use
+** as a list terminator.
+**
+** zWrapperId is an optional ID value for the containing element (see
+** below).
+**
+** zFieldName is the value of the form element's name attribute. Note
+** that fossil prefers underscores over '-' for separators in form
+** element names.
+**
+** zLabel is an optional string to use as a "label" for the element
+** (see below).
+**
+** zTooltip is an optional value for the SELECT's title attribute.
+**
+** The structure of the emitted HTML is:
+**
+** <div class='input-with-label' title={{zToolTip}} id={{zWrapperId}}>
+**   <label for='SELECT ELEMENT ID'>{{zLabel}}</label>
+**   <select id='RANDOM ID' name={{zFieldName}}>...</select>
+** </div>
+**
+** Example:
+**
+** style_select_list_int("my-grapes", "my_grapes", "Grapes",
+**                      "Select the number of grapes",
+**                       atoi(PD("my_field","0")),
+**                       "", 1, "2", 2, "Three", 3,
+**                       NULL);
+** 
+*/
+void style_select_list_int(const char * zWrapperId,
+                           const char *zFieldName, const char * zLabel,
+                           const char * zToolTip, int selectedVal,
+                           ... ){
+  char * zLabelID = style_next_input_id();
+  va_list vargs;
+
+  va_start(vargs,selectedVal);
+  CX("<div class='input-with-label'");
+  if(zToolTip && *zToolTip){
+    CX(" title='%h'",zToolTip);
+  }
+  if(zWrapperId && *zWrapperId){
+    CX(" id='%s'",zWrapperId);
+  }
+  CX(">");
+  if(zLabel && *zLabel){
+    CX("<label for='%s'>%h</label>", zLabelID, zLabel);
+  }
+  CX("<select name='%s' id='%s'>",zFieldName, zLabelID);
+  while(1){
+    const char * zOption = va_arg(vargs,char *);
+    int v;
+    if(NULL==zOption){
+      break;
+    }
+    v = va_arg(vargs,int);
+    CX("<option value='%d'%s>",
+         v, v==selectedVal ? " selected" : "");
+    if(*zOption){
+      CX("%s", zOption);
+    }else{
+      CX("%d",v);
+    }
+    CX("</option>\n");
+  }
+  CX("</select>\n");
+  CX("</div>\n");
+  va_end(vargs);
+  fossil_free(zLabelID);
+}
+
+/*
+** The C-string counterpart of style_select_list_int(), this variant
+** differs only in that its variadic arguments are C-strings in pairs
+** of (optionLabel, optionValue). If a given optionLabel is an empty
+** string, the corresponding optionValue is used as its label. If any
+** given value matches zSelectedVal, that option gets preselected. If
+** no options match zSelectedVal then the first entry is selected by
+** default.
+**
+** Any of (zWrapperId, zTooltip, zSelectedVal) may be NULL or empty.
+**
+** Example:
+**
+** style_select_list_str("my-grapes", "my_grapes", "Grapes",
+**                      "Select the number of grapes",
+**                       P("my_field"),
+**                       "1", "One", "2", "Two", "", "3",
+**                       NULL);
+*/
+void style_select_list_str(const char * zWrapperId,
+                           const char *zFieldName, const char * zLabel,
+                           const char * zToolTip, char const * zSelectedVal,
+                           ... ){
+  char * zLabelID = style_next_input_id();
+  va_list vargs;
+
+  va_start(vargs,zSelectedVal);
+  if(!zSelectedVal){
+    zSelectedVal = __FILE__/*some string we'll never match*/;
+  }
+  CX("<div class='input-with-label'");
+  if(zToolTip && *zToolTip){
+    CX(" title='%h'",zToolTip);
+  }
+  if(zWrapperId && *zWrapperId){
+    CX(" id='%s'",zWrapperId);
+  }
+  CX(">");
+  if(zLabel && *zLabel){
+    CX("<label for='%s'>%h</label>", zLabelID, zLabel);
+  }
+  CX("<select name='%s' id='%s'>",zFieldName, zLabelID);
+  while(1){
+    const char * zLabel = va_arg(vargs,char *);
+    const char * zVal;
+    if(NULL==zLabel){
+      break;
+    }
+    zVal = va_arg(vargs,char *);
+    CX("<option value='%T'%s>",
+       zVal, 0==fossil_strcmp(zVal, zSelectedVal) ? " selected" : "");
+    if(*zLabel){
+      CX("%s", zLabel);
+    }else{
+      CX("%h",zVal);
+    }
+    CX("</option>\n");
+  }
+  CX("</select>\n");
+  CX("</div>\n");
+  va_end(vargs);
+  fossil_free(zLabelID);
+}
+
+/*
+** Generate a <script> with an appropriate nonce.
+**
+** zOrigin and iLine are the source code filename and line number
+** that generated this request.
+*/
+void style_script_begin(const char *zOrigin, int iLine){
+  const char *z;
+  for(z=zOrigin; z[0]!=0; z++){
+    if( z[0]=='/' || z[0]=='\\' ){
+      zOrigin = z+1;
+    }
+  }
+  CX("<script nonce='%s'>/* %s:%d */\n", style_nonce(), zOrigin, iLine);
+}
+
+/* Generate the closing </script> tag 
+*/
+void style_script_end(void){
+  CX("</script>\n");
+}
+
+/*
+** Emits a NOSCRIPT tag with an error message stating that JS is
+** required for the current page. This "should" be called near the top
+** of pages which *require* JS. The inner DIV has the CSS class
+** 'error' and can be styled via a (noscript > .error) CSS selector.
+*/
+void style_emit_noscript_for_js_page(void){
+  CX("<noscript><div class='error'>"
+     "This page requires JavaScript (ES2015, a.k.a. ES6, or newer)."
+     "</div></noscript>");
+}
