@@ -9,9 +9,30 @@
 #include <string.h>
 #include <assert.h>
 
-typedef struct Th_Command   Th_Command;
-typedef struct Th_Frame     Th_Frame;
-typedef struct Th_Variable  Th_Variable;
+/*
+** Values used for element values in the tcl_platform array.
+*/
+
+#if !defined(TH_ENGINE)
+#  define TH_ENGINE          "TH1"
+#endif
+
+#if !defined(TH_PLATFORM)
+#  if defined(_WIN32) || defined(WIN32)
+#    define TH_PLATFORM      "windows"
+#  else
+#    define TH_PLATFORM      "unix"
+#  endif
+#endif
+
+/*
+** Forward declarations for structures defined below.
+*/
+
+typedef struct Th_Command        Th_Command;
+typedef struct Th_Frame          Th_Frame;
+typedef struct Th_Variable       Th_Variable;
+typedef struct Th_InterpAndList  Th_InterpAndList;
 
 /*
 ** Interpreter structure.
@@ -90,6 +111,17 @@ struct Th_Variable {
   Th_Hash *pHash;             /* Data for array variables */
 };
 
+/*
+** This structure is used to pass complete context information to the
+** hash iteration callback functions that need a Th_Interp and a list
+** to operate on, e.g. thListAppendHashKey().
+*/
+struct Th_InterpAndList {
+  Th_Interp *interp;          /* Associated interpreter context */
+  char **pzList;              /* IN/OUT: Ptr to ptr to list */
+  int *pnList;                /* IN/OUT: Current length of *pzList */
+};
+
 /*
 ** Hash table API:
 */
@@ -173,6 +205,14 @@ static int  thBufferWrite(Th_Interp *interp, Buffer *, const char *, int);
 static void thBufferInit(Buffer *);
 static void thBufferFree(Th_Interp *interp, Buffer *);
 
+/*
+** This version of memcpy() allows the first and second argument to
+** be NULL as long as the number of bytes to copy is zero.
+*/
+static void *th_memcpy(void *dest, const void *src, size_t n){
+  return n>0 ? memcpy(dest,src,n) : dest;
+}
+
 /*
 ** Append nAdd bytes of content copied from zAdd to the end of buffer
 ** pBuffer. If there is not enough space currently allocated, resize
@@ -197,13 +237,13 @@ static int thBufferWrite(
 
     nNew = nReq*2;
     zNew = (char *)Th_Malloc(interp, nNew);
-    memcpy(zNew, pBuffer->zBuf, pBuffer->nBuf);
+    th_memcpy(zNew, pBuffer->zBuf, pBuffer->nBuf);
     Th_Free(interp, pBuffer->zBuf);
     pBuffer->nBufAlloc = nNew;
     pBuffer->zBuf = zNew;
   }
 
-  memcpy(&pBuffer->zBuf[pBuffer->nBuf], zAdd, nAdd);
+  th_memcpy(&pBuffer->zBuf[pBuffer->nBuf], zAdd, nAdd);
   pBuffer->nBuf += nAdd;
   pBuffer->zBuf[pBuffer->nBuf] = '\0';
 
@@ -301,6 +341,21 @@ static int thFreeCommand(Th_HashEntry *pEntry, void *pContext){
   return 1;
 }
 
+/*
+** Argument pEntry points to an entry in a hash table.  The key is
+** the list element to be added.
+**
+** Argument pContext is a pointer to the Th_InterpAndList structure.
+**
+** Always returns non-zero.
+*/
+static int thListAppendHashKey(Th_HashEntry *pEntry, void *pContext){
+  Th_InterpAndList *pInterpAndList = (Th_InterpAndList *)pContext;
+  Th_ListAppend(pInterpAndList->interp, pInterpAndList->pzList,
+                pInterpAndList->pnList, pEntry->zKey, pEntry->nKey);
+  return 1;
+}
+
 /*
 ** Push a new frame onto the stack.
 */
@@ -796,8 +851,8 @@ static int thSplitList(
     );
     anElem = (int *)&azElem[nCount];
     zElem = (char *)&anElem[nCount];
-    memcpy(anElem, lenbuf.zBuf, lenbuf.nBuf);
-    memcpy(zElem, strbuf.zBuf, strbuf.nBuf);
+    th_memcpy(anElem, lenbuf.zBuf, lenbuf.nBuf);
+    th_memcpy(zElem, strbuf.zBuf, strbuf.nBuf);
     for(i=0; i<nCount;i++){
       azElem[i] = zElem;
       zElem += (anElem[i] + 1);
@@ -1081,7 +1136,7 @@ typedef struct Find Find;
 **
 ** If the arrayok argument is false and the named variable is an array,
 ** an error is left in the interpreter result and NULL returned. If
-** arrayok is true an array name is Ok.
+** arrayok is true an array name is OK.
 */
 
 static Th_Variable *thFindValue(
@@ -1089,7 +1144,7 @@ static Th_Variable *thFindValue(
   const char *zVar,       /* Pointer to variable name */
   int nVar,               /* Number of bytes at nVar */
   int create,             /* If true, create the variable if not found */
-  int arrayok,            /* If true, an array is Ok. Otherwise array==error */
+  int arrayok,            /* If true, an array is OK. Otherwise array==error */
   int noerror,            /* If false, set interpreter result to error */
   Find *pFind             /* If non-zero, place output here */
 ){
@@ -1207,6 +1262,14 @@ int Th_ExistsVar(Th_Interp *interp, const char *zVar, int nVar){
   return pValue && (pValue->zData || pValue->pHash);
 }
 
+/*
+** Return true if array variable (zVar, nVar) exists.
+*/
+int Th_ExistsArrayVar(Th_Interp *interp, const char *zVar, int nVar){
+  Th_Variable *pValue = thFindValue(interp, zVar, nVar, 0, 1, 1, 0);
+  return pValue && !pValue->zData && pValue->pHash;
+}
+
 /*
 ** String (zVar, nVar) must contain the name of a scalar variable or
 ** array member. If the variable does not exist it is created. The
@@ -1240,7 +1303,7 @@ int Th_SetVar(
   assert(zValue || nValue==0);
   pValue->zData = Th_Malloc(interp, nValue+1);
   pValue->zData[nValue] = '\0';
-  memcpy(pValue->zData, zValue, nValue);
+  th_memcpy(pValue->zData, zValue, nValue);
   pValue->nData = nValue;
 
   return TH_OK;
@@ -1359,7 +1422,7 @@ char *th_strdup(Th_Interp *interp, const char *z, int n){
     n = th_strlen(z);
   }
   zRes = Th_Malloc(interp, n+1);
-  memcpy(zRes, z, n);
+  th_memcpy(zRes, z, n);
   zRes[n] = '\0';
   return zRes;
 }
@@ -1419,7 +1482,7 @@ int Th_SetResult(Th_Interp *pInterp, const char *z, int n){
   if( z && n>0 ){
     char *zResult;
     zResult = Th_Malloc(pInterp, n+1);
-    memcpy(zResult, z, n);
+    th_memcpy(zResult, z, n);
     zResult[n] = '\0';
     pInterp->zResult = zResult;
     pInterp->nResult = n;
@@ -1722,8 +1785,8 @@ int Th_StringAppend(
 
   nNew = *pnStr + nElem;
   zNew = Th_Malloc(interp, nNew);
-  memcpy(zNew, *pzStr, *pnStr);
-  memcpy(&zNew[*pnStr], zElem, nElem);
+  th_memcpy(zNew, *pzStr, *pnStr);
+  th_memcpy(&zNew[*pnStr], zElem, nElem);
 
   Th_Free(interp, *pzStr);
   *pzStr = zNew;
@@ -1732,6 +1795,18 @@ int Th_StringAppend(
   return TH_OK;
 }
 
+/*
+** Initialize an interpreter.
+*/
+static int thInitialize(Th_Interp *interp){
+  assert(interp->pFrame);
+
+  Th_SetVar(interp, (char *)"::tcl_platform(engine)", -1, TH_ENGINE, -1);
+  Th_SetVar(interp, (char *)"::tcl_platform(platform)", -1, TH_PLATFORM, -1);
+
+  return TH_OK;
+}
+
 /*
 ** Delete an interpreter.
 */
@@ -1765,6 +1840,7 @@ Th_Interp * Th_CreateInterp(Th_Vtab *pVtab){
   p->pVtab = pVtab;
   p->paCmd = Th_HashNew(p);
   thPushFrame(p, (Th_Frame *)&p[1]);
+  thInitialize(p);
 
   return p;
 }
@@ -2229,14 +2305,25 @@ static int exprParse(
           const char *zOp;
           for(j=0; (zOp=aOperator[j].zOp); j++){
             int nOp = aOperator[j].nOp;
+            int nRemain = nExpr - i;
             int isMatch = 0;
-            if( (nExpr-i)>=nOp && 0==memcmp(zOp, &zExpr[i], nOp) ){
+            if( nRemain>=nOp && 0==memcmp(zOp, &zExpr[i], nOp) ){
               isMatch = 1;
             }
-            if( isMatch && aOperator[j].eOp==OP_OPEN_BRACKET ){
-              nNest++;
-            }else if( isMatch && aOperator[j].eOp==OP_CLOSE_BRACKET ){
-              nNest--;
+            if( isMatch ){
+              if( aOperator[j].eOp==OP_CLOSE_BRACKET ){
+                nNest--;
+              }else if( nRemain>nOp ){
+                if( aOperator[j].eOp==OP_OPEN_BRACKET ){
+                  nNest++;
+                }
+              }else{
+                /*
+                ** This is not really a match because this operator cannot
+                ** legally appear at the end of the string.
+                */
+                isMatch = 0;
+              }
             }
             if( nToken>0 && aOperator[j].iPrecedence==1 ){
               Expr *pPrev = apToken[nToken-1];
@@ -2258,14 +2345,14 @@ static int exprParse(
           /* A terminal. Copy the string value. */
           assert( !pNew->pOp );
           pNew->zValue = Th_Malloc(interp, pNew->nValue);
-          memcpy(pNew->zValue, z, pNew->nValue);
+          th_memcpy(pNew->zValue, z, pNew->nValue);
           i += pNew->nValue;
         }
         if( (nToken%16)==0 ){
           /* Grow the apToken array. */
           Expr **apTokenOld = apToken;
           apToken = Th_Malloc(interp, sizeof(Expr *)*(nToken+16));
-          memcpy(apToken, apTokenOld, sizeof(Expr *)*nToken);
+          th_memcpy(apToken, apTokenOld, sizeof(Expr *)*nToken);
         }
 
         /* Put the new token at the end of the apToken array */
@@ -2436,7 +2523,7 @@ Th_HashEntry *Th_HashFind(
     pRet = (Th_HashEntry *)Th_Malloc(interp, sizeof(Th_HashEntry) + nKey);
     pRet->zKey = (char *)&pRet[1];
     pRet->nKey = nKey;
-    memcpy(pRet->zKey, zKey, nKey);
+    th_memcpy(pRet->zKey, zKey, nKey);
     pRet->pNext = pHash->a[iKey];
     pHash->a[iKey] = pRet;
   }
@@ -2648,24 +2735,22 @@ int Th_ToInt(Th_Interp *interp, const char *z, int n, int *piOut){
     n = th_strlen(z);
   }
 
-  if( n>0 && (z[0]=='-' || z[0]=='+') ){
+  if( n>1 && (z[0]=='-' || z[0]=='+') ){
     i = 1;
   }
-  if( n>2 ){
-    if( z[i]=='0' ){
-      if( z[i+1]=='x' || z[i+1]=='X' ){
-        i += 2;
-        base = 16;
-        isdigit = th_ishexdig;
-      }else if( z[i+1]=='o' || z[i+1]=='O' ){
-        i += 2;
-        base = 8;
-        isdigit = th_isoctdig;
-      }else if( z[i+1]=='b' || z[i+1]=='B' ){
-        i += 2;
-        base = 2;
-        isdigit = th_isbindig;
-      }
+  if( (n-i)>2 && z[i]=='0' ){
+    if( z[i+1]=='x' || z[i+1]=='X' ){
+      i += 2;
+      base = 16;
+      isdigit = th_ishexdig;
+    }else if( z[i+1]=='o' || z[i+1]=='O' ){
+      i += 2;
+      base = 8;
+      isdigit = th_isoctdig;
+    }else if( z[i+1]=='b' || z[i+1]=='B' ){
+      i += 2;
+      base = 2;
+      isdigit = th_isbindig;
     }
   }
   for(; i<n; i++){
@@ -2825,3 +2910,79 @@ int Th_SetResultDouble(Th_Interp *interp, double fVal){
   *z = '\0';
   return Th_SetResult(interp, zBuf, -1);
 }
+
+/*
+** Appends all currently registered command names to the specified list
+** and returns TH_OK upon success.  Any other return value indicates an
+** error.
+*/
+int Th_ListAppendCommands(
+  Th_Interp *interp,      /* Interpreter context */
+  char **pzList,          /* OUT: List of command names */
+  int *pnList             /* OUT: Number of command names */
+){
+  Th_InterpAndList *p = (Th_InterpAndList *)Th_Malloc(
+    interp, sizeof(Th_InterpAndList)
+  );
+  p->interp = interp;
+  p->pzList = pzList;
+  p->pnList = pnList;
+  Th_HashIterate(interp, interp->paCmd, thListAppendHashKey, p);
+  Th_Free(interp, p);
+  return TH_OK;
+}
+
+/*
+** Appends all variable names for the current frame to the specified list
+** and returns TH_OK upon success.  Any other return value indicates an
+** error.  If the current frame cannot be obtained, TH_ERROR is returned.
+*/
+int Th_ListAppendVariables(
+  Th_Interp *interp,      /* Interpreter context */
+  char **pzList,          /* OUT: List of variable names */
+  int *pnList             /* OUT: Number of variable names */
+){
+  Th_Frame *pFrame = getFrame(interp, 0);
+  if( pFrame ){
+    Th_InterpAndList *p = (Th_InterpAndList *)Th_Malloc(
+      interp, sizeof(Th_InterpAndList)
+    );
+    p->interp = interp;
+    p->pzList = pzList;
+    p->pnList = pnList;
+    Th_HashIterate(interp, pFrame->paVar, thListAppendHashKey, p);
+    Th_Free(interp, p);
+    return TH_OK;
+  }else{
+    return TH_ERROR;
+  }
+}
+
+/*
+** Appends all array element names for the specified array variable to the
+** specified list and returns TH_OK upon success.  Any other return value
+** indicates an error.
+*/
+int Th_ListAppendArray(
+  Th_Interp *interp,      /* Interpreter context */
+  const char *zVar,       /* Pointer to variable name */
+  int nVar,               /* Number of bytes at nVar */
+  char **pzList,          /* OUT: List of array element names */
+  int *pnList             /* OUT: Number of array element names */
+){
+  Th_Variable *pValue = thFindValue(interp, zVar, nVar, 0, 1, 1, 0);
+  if( pValue && !pValue->zData && pValue->pHash ){
+    Th_InterpAndList *p = (Th_InterpAndList *)Th_Malloc(
+      interp, sizeof(Th_InterpAndList)
+    );
+    p->interp = interp;
+    p->pzList = pzList;
+    p->pnList = pnList;
+    Th_HashIterate(interp, pValue->pHash, thListAppendHashKey, p);
+    Th_Free(interp, p);
+  }else{
+    *pzList = 0;
+    *pnList = 0;
+  }
+  return TH_OK;
+}
