@@ -258,6 +258,9 @@ static int contains_whitespace(const char *zName){
 **
 **                              --glob PATTERN   Show only files that match
 **                              --like PATTERN   Show only files that match
+**                              -l               Show additional details for
+**                                               files that match. Implied
+**                                               when 'list' is used.
 **
 **    revert ?URL?           Restore the state of all unversioned files in the
 **                           local repository to match the remote repository
@@ -379,7 +382,7 @@ void unversioned_cmd(void){
     blob_add_cr(&content);
 #endif
     blob_write_to_file(&content, zTFile);
-    zCmd = mprintf("%s \"%s\"", zEditor, zTFile);
+    zCmd = mprintf("%s %$", zEditor, zTFile);
     if( fossil_system(zCmd) ){
       fossil_fatal("editor aborted: %Q", zCmd);
     }
