@@ -1,58 +1,95 @@
-#!/bin/sh
+#!/usr/bin/env tclsh
 #
 # Run this TCL script to generate a WIKI page that contains a
 # permuted index of the various documentation files.
 #
-#    tclsh mkindex.tcl 
+#    tclsh mkindex.tcl
 #
 
 set doclist {
+  aboutcgi.wiki {How CGI Works In Fossil}
+  aboutdownload.wiki {How The Download Page Works}
   adding_code.wiki {Adding New Features To Fossil}
   adding_code.wiki {Hacking Fossil}
+  alerts.md {Email Alerts And Notifications}
   antibot.wiki {Defense against Spiders and Bots}
+  backoffice.md {The "Backoffice" mechanism of Fossil}
+  backup.md {Backing Up a Remote Fossil Repository}
   blame.wiki {The Annotate/Blame Algorithm Of Fossil}
+  blockchain.md {Is Fossil A Blockchain?}
   branching.wiki {Branching, Forking, Merging, and Tagging}
   bugtheory.wiki {Bug Tracking In Fossil}
   build.wiki {Compiling and Installing Fossil}
+  cap-theorem.md {Fossil and the CAP Theorem}
+  caps/ {Administering User Capabilities}
+  caps/admin-v-setup.md {Differences Between Setup and Admin Users}
+  caps/ref.html {User Capability Reference}
+  cgi.wiki {CGI Script Configuration Options}
+  changes.wiki {Fossil Changelog}
   checkin_names.wiki {Check-in And Version Names}
   checkin.wiki {Check-in Checklist}
-  changes.wiki {Fossil Changelog}
+  childprojects.wiki {Child Projects}
   copyright-release.html {Contributor License Agreement}
   concepts.wiki {Fossil Core Concepts}
   contribute.wiki {Contributing Code or Documentation To The Fossil Project}
+  css-tricks.md {Fossil CSS Tips and Tricks}
   customgraph.md {Theming: Customizing the Timeline Graph}
   customskin.md {Theming: Customizing The Appearance of Web Pages}
+  customskin.md {Custom Skins}
   custom_ticket.wiki {Customizing The Ticket System}
+  defcsp.md {The Default Content Security Policy}
   delta_encoder_algorithm.wiki {Fossil Delta Encoding Algorithm}
   delta_format.wiki {Fossil Delta Format}
   embeddeddoc.wiki {Embedded Project Documentation}
+  encryptedrepos.wiki {How To Use Encrypted Repositories}
   env-opts.md {Environment Variables and Global Options}
   event.wiki {Events}
   faq.wiki {Frequently Asked Questions}
+  fileedit-page.md {The fileedit Page}
   fileformat.wiki {Fossil File Format}
-  fiveminutes.wiki {Update and Running in 5 Minutes as a Single User}
+  fiveminutes.wiki {Up and Running in 5 Minutes as a Single User}
+  forum.wiki {Fossil Forums}
   foss-cklist.wiki {Checklist For Successful Open-Source Projects}
   fossil-from-msvc.wiki {Integrating Fossil in the Microsoft Express 2010 IDE}
+  fossil_prompt.wiki {Fossilized Bash Prompt}
   fossil-v-git.wiki {Fossil Versus Git}
+  globs.md {File Name Glob Patterns}
+  gitusers.md {Hints For Users With Git Experience}
+  grep.md {Fossil grep vs POSIX grep}
   hacker-howto.wiki {Hacker How-To}
+  hacker-howto.wiki {Fossil Developers Guide}
+  hashes.md {Hashes: Fossil Artifact Identification}
+  hashpolicy.wiki {Hash Policy: Choosing Between SHA1 and SHA3-256}
+  /help {Lists of Commands and Webpages}
   hints.wiki {Fossil Tips And Usage Hints}
+  history.md {The Purpose And History Of Fossil}
   index.wiki {Home Page}
   inout.wiki {Import And Export To And From Git}
+  interwiki.md {Interwiki Links}
+  image-format-vs-repo-size.md {Image Format vs Fossil Repo Size}
+  javascript.md {Use of JavaScript in Fossil}
   makefile.wiki {The Fossil Build Process}
+  mirrorlimitations.md {Limitations On Git Mirrors}
+  mirrortogithub.md {How To Mirror A Fossil Repository On GitHub}
+  /md_rules {Markdown Formatting Rules}
   newrepo.wiki {How To Create A New Fossil Repository}
   password.wiki {Password Management And Authentication}
-  pop.wiki {Principles Of Operations}
+  pop.wiki {Principles Of Operation}
   private.wiki {Creating, Syncing, and Deleting Private Branches}
   qandc.wiki {Questions And Criticisms}
   quickstart.wiki {Fossil Quick Start Guide}
   quotes.wiki
       {Quotes: What People Are Saying About Fossil, Git, and DVCSes in General}
   ../test/release-checklist.wiki {Pre-Release Testing Checklist}
+  rebaseharm.md {Rebase Considered Harmful}
   reviews.wiki {Reviews}
   selfcheck.wiki {Fossil Repository Integrity Self Checks}
   selfhost.wiki {Fossil Self Hosting Repositories}
-  server.wiki {How To Configure A Fossil Server}
+  server/ {How To Configure A Fossil Server}
+  serverext.wiki {CGI Server Extensions}
+  serverext.wiki {Adding Extensions To A Fossil Server Using CGI Scripts}
   settings.wiki {Fossil Settings}
+  /sitemap {Site Map}
   shunning.wiki {Shunning: Deleting Content From Fossil}
   stats.wiki {Performance Statistics}
   style.wiki {Source Code Style Guidelines}
@@ -64,23 +101,31 @@ set doclist {
   th1.md {The TH1 Scripting Language}
   tickets.wiki {The Fossil Ticket System}
   theory1.wiki {Thoughts On The Design Of The Fossil DVCS}
+  tls-nginx.md {Proxying Fossil via HTTPS with nginx}
+  unvers.wiki {Unversioned Files}
   webpage-ex.md {Webpage Examples}
   webui.wiki {The Fossil Web Interface}
+  whyusefossil.wiki {Why You Should Use Fossil}
+  whyusefossil.wiki {Benefits Of Version Control}
   wikitheory.wiki {Wiki In Fossil}
+  /wiki_rules {Wiki Formatting Rules}
 }
 
 set permindex {}
-set stopwords {fossil and a in of on the to are about used by for or}
+set stopwords {
+   a about against and are as by for fossil from in of on or should the to use
+   used with
+}
 foreach {file title} $doclist {
   set n [llength $title]
   regsub -all {\s+} $title { } title
-  lappend permindex [list $title $file]
+  lappend permindex [list $title $file 1]
   for {set i 0} {$i<$n-1} {incr i} {
     set prefix [lrange $title 0 $i]
     set suffix [lrange $title [expr {$i+1}] end]
     set firstword [string tolower [lindex $suffix 0]]
     if {[lsearch $stopwords $firstword]<0} {
-      lappend permindex [list "$suffix &mdash; $prefix" $file]
+      lappend permindex [list "$suffix &mdash; $prefix" $file 0]
     }
   }
 }
@@ -99,18 +144,22 @@ puts $out {
 <h2>Primary Documents:</h2>
 <ul>
 <li> <a href='quickstart.wiki'>Quick-start Guide</a>
-<li> <a href='faq.wiki'>FAQ</a>
+<li> <a href='$ROOT/help'>Built-in help for commands and webpages</a>
+<li> <a href='history.md'>Purpose and History of Fossil</a>
 <li> <a href='build.wiki'>Compiling and installing Fossil</a>
 <li> <a href='../COPYRIGHT-BSD2.txt'>License</a>
+<li> <a href='userlinks.wiki'>Miscellaneous Docs for Fossil Users</a>
+<li> <a href='hacker-howto.wiki'>Fossil Developer's Guide</a>
 <li> <a href='http://www.fossil-scm.org/schimpf-book/home'>Jim Schimpf's
 book</a>
-<li> <a href='$ROOT/help'>Command-line help</a>
 </ul>
 <a name="pindex"></a>
 <h2>Permuted Index:</h2>
 <ul>}
 foreach entry $permindex {
-  foreach {title file} $entry break
+  foreach {title file bold} $entry break
+  if {$bold} {set title <b>$title</b>}
+  if {[string match /* $file]} {set file ../../..$file}
   puts $out "<li><a href=\"$file\">$title</a></li>"
 }
 puts $out "</ul></div>"
